@@ -102,6 +102,35 @@ void KoShapeManager::Private::updateTree()
     }
 }
 
+void KoShapeManager::Private::forwardCompressedUdpate()
+{
+    bool shouldUpdateDecorations = false;
+    QRectF scheduledUpdate;
+
+    {
+        QMutexLocker l(&shapesMutex);
+
+        if (!compressedUpdate.isEmpty()) {
+            scheduledUpdate = compressedUpdate;
+            compressedUpdate = QRect();
+        }
+
+        Q_FOREACH (const KoShape *shape, compressedUpdatedShapes) {
+            if (selection->isSelected(shape)) {
+                shouldUpdateDecorations = true;
+                break;
+            }
+        }
+        compressedUpdatedShapes.clear();
+    }
+
+    if (shouldUpdateDecorations && canvas->toolProxy()) {
+        canvas->toolProxy()->repaintDecorations();
+    }
+    canvas->updateCanvas(scheduledUpdate);
+
+}
+
 void KoShapeManager::Private::paintGroup(KoShapeGroup *group, QPainter &painter, const KoViewConverter &converter, KoShapePaintingContext &paintContext)
 {
     QList<KoShape*> shapes = group->shapes();
@@ -134,6 +163,7 @@ KoShapeManager::KoShapeManager(KoCanvasBase *canvas, const QList<KoShape *> &sha
      * to the GUI thread.
      */
     this->moveToThread(qApp->thread());
+    connect(&d->updateCompressor, SIGNAL(timeout()), this, SLOT(forwardCompressedUdpate()));
 }
 
 KoShapeManager::KoShapeManager(KoCanvasBase *canvas)
@@ -144,6 +174,7 @@ KoShapeManager::KoShapeManager(KoCanvasBase *canvas)
 
     // see a comment in another constructor
     this->moveToThread(qApp->thread());
+    connect(&d->updateCompressor, SIGNAL(timeout()), this, SLOT(forwardCompressedUdpate()));
 }
 
 void KoShapeManager::Private::unlinkFromShapesRecursively(const QList<KoShape*> &shapes)
@@ -175,6 +206,8 @@ void KoShapeManager::setShapes(const QList<KoShape *> &shapes, Repaint repaint)
         //clear selection
         d->selection->deselectAll();
         d->unlinkFromShapesRecursively(d->shapes);
+        d->compressedUpdate = QRect();
+        d->compressedUpdatedShapes.clear();
         d->aggregate4update.clear();
         d->shapeIndexesBeforeUpdate.clear();
         d->tree.clear();
@@ -242,6 +275,7 @@ void KoShapeManager::remove(KoShape *shape)
 
         d->selection->deselect(shape);
         d->aggregate4update.remove(shape);
+        d->compressedUpdatedShapes.remove(shape);
 
         if (d->shapeUsedInRenderingTree(shape)) {
             d->tree.remove(shape);
@@ -274,6 +308,7 @@ void KoShapeManager::ShapeInterface::notifyShapeDestructed(KoShape *shape)
 
     q->d->selection->deselect(shape);
     q->d->aggregate4update.remove(shape);
+    q->d->compressedUpdatedShapes.remove(shape);
 
     // we cannot access RTTI of the semi-destructed shape, so just
     // unlink it lazily
@@ -626,15 +661,18 @@ QList<KoShape *> KoShapeManager::shapesAt(const QRectF &rect, bool omitHiddenSha
 
 void KoShapeManager::update(const QRectF &rect, const KoShape *shape, bool selectionHandles)
 {
-    // TODO: do we need locking here?
+    {
+        QMutexLocker l(&d->shapesMutex);
 
-    d->canvas->updateCanvas(rect);
-    if (selectionHandles && d->selection->isSelected(shape)) {
-        if (d->canvas->toolProxy())
-            d->canvas->toolProxy()->repaintDecorations();
+        d->compressedUpdate |= rect;
+
+        if (selectionHandles) {
+            d->compressedUpdatedShapes.insert(shape);
+        }
     }
-}
 
+    d->updateCompressor.start();
+}
 void KoShapeManager::notifyShapeChanged(KoShape *shape)
 {
     {
