@@ -43,12 +43,12 @@ typedef KisSafeReadList<KisNodeSP> KisSafeReadNodeList;
 #include "kis_abstract_projection_plane.h"
 #include "kis_projection_leaf.h"
 #include "kis_undo_adapter.h"
-
 #include "kis_keyframe_channel.h"
-#include "kis_time_range.h"
+#include "kis_image.h"
+#include "kis_layer_utils.h"
 
 /**
- *The link between KisProjection ans KisImageUpdater
+ *The link between KisProjection and KisImageUpdater
  *uses queued signals with an argument of KisNodeSP type,
  *so we should register it beforehand
  */
@@ -94,8 +94,6 @@ public:
             : graphListener(0)
             , nodeProgressProxy(0)
             , busyProgressIndicator(0)
-            , animated(false)
-            , useInTimeline(false)
             , projectionLeaf(new KisProjectionLeaf(node))
     {
     }
@@ -106,9 +104,6 @@ public:
     KisNodeProgressProxy *nodeProgressProxy;
     KisBusyProgressIndicator *busyProgressIndicator;
     QReadWriteLock nodeSubgraphLock;
-    QMap<QString, KisKeyframeChannel*> keyframeChannels;
-    bool animated;
-    bool useInTimeline;
 
     KisProjectionLeafSP projectionLeaf;
 
@@ -167,7 +162,7 @@ void KisNode::Private::processDuplicatedClones(const KisNode *srcDuplicationRoot
                                                         clone->copyFrom());
 
         if (newCopyFrom) {
-            KisLayer *newCopyFromLayer = dynamic_cast<KisLayer*>(const_cast<KisNode*>(newCopyFrom));
+            KisLayer *newCopyFromLayer = qobject_cast<KisLayer*>(const_cast<KisNode*>(newCopyFrom));
             KIS_ASSERT_RECOVER_RETURN(newCopyFromLayer);
 
             clone->setCopyFrom(newCopyFromLayer);
@@ -181,8 +176,9 @@ void KisNode::Private::processDuplicatedClones(const KisNode *srcDuplicationRoot
     }
 }
 
-KisNode::KisNode()
-        : m_d(new Private(this))
+KisNode::KisNode(KisImageWSP image)
+        : KisBaseNode(image),
+          m_d(new Private(this))
 {
     m_d->parent = 0;
     m_d->graphListener = 0;
@@ -197,6 +193,13 @@ KisNode::KisNode(const KisNode & rhs)
     m_d->graphListener = 0;
     moveToThread(qApp->thread());
 
+    // HACK ALERT: we create opacity channel in KisBaseNode, but we cannot
+    //             initialize its node from there! So workaround it here!
+    QMap<QString, KisKeyframeChannel*> channels = keyframeChannels();
+    for (auto it = channels.begin(); it != channels.end(); ++it) {
+        it.value()->setNode(this);
+    }
+
     // NOTE: the nodes are not supposed to be added/removed while
     // creation of another node, so we do *no* locking here!
 
@@ -214,11 +217,12 @@ KisNode::KisNode(const KisNode & rhs)
 KisNode::~KisNode()
 {
     if (m_d->busyProgressIndicator) {
-        m_d->busyProgressIndicator->endUpdatesBeforeDestroying();
+        m_d->busyProgressIndicator->prepareDestroying();
         m_d->busyProgressIndicator->deleteLater();
     }
 
     if (m_d->nodeProgressProxy) {
+        m_d->nodeProgressProxy->prepareDestroying();
         m_d->nodeProgressProxy->deleteLater();
     }
 
@@ -248,6 +252,10 @@ QRect KisNode::accessRect(const QRect &rect, PositionToFilthy pos) const
     return rect;
 }
 
+void KisNode::childNodeChanged(KisNodeSP /*changedChildNode*/)
+{
+}
+
 KisAbstractProjectionPlaneSP KisNode::projectionPlane() const
 {
     KIS_ASSERT_RECOVER_NOOP(0 && "KisNode::projectionPlane() is not defined!");
@@ -257,52 +265,26 @@ KisAbstractProjectionPlaneSP KisNode::projectionPlane() const
     return plane;
 }
 
-QList<KisKeyframeChannel*> KisNode::keyframeChannels() const
-{
-    return m_d->keyframeChannels.values();
-}
-
-KisKeyframeChannel * KisNode::getKeyframeChannel(const QString &id) const
-{
-    QMap<QString, KisKeyframeChannel*>::iterator i = m_d->keyframeChannels.find(id);
-    if (i == m_d->keyframeChannels.end()) return 0;
-    return i.value();
-}
-
-bool KisNode::isAnimated() const
-{
-    return m_d->animated;
-}
-
-void KisNode::enableAnimation()
-{
-    m_d->animated = true;
-    baseNodeChangedCallback();
-}
-
-bool KisNode::useInTimeline() const
-{
-    return m_d->useInTimeline;
-}
-
-void KisNode::setUseInTimeline(bool value)
-{
-    if (value == m_d->useInTimeline) return;
-
-    m_d->useInTimeline = value;
-    baseNodeChangedCallback();
-}
-
-void KisNode::addKeyframeChannel(KisKeyframeChannel *channel)
-{
-    m_d->keyframeChannels.insert(channel->id(), channel);
-}
-
 KisProjectionLeafSP KisNode::projectionLeaf() const
 {
     return m_d->projectionLeaf;
 }
 
+void KisNode::setImage(KisImageWSP image)
+{
+    KisBaseNode::setImage(image);
+
+    KisNodeSP node = firstChild();
+    while (node) {
+        KisLayerUtils::recursiveApplyNodes(node,
+                                           [image] (KisNodeSP node) {
+                                               node->setImage(image);
+                                           });
+
+        node = node->nextSibling();
+    }
+}
+
 bool KisNode::accept(KisNodeVisitor &v)
 {
     return v.visit(this);
@@ -310,7 +292,7 @@ bool KisNode::accept(KisNodeVisitor &v)
 
 void KisNode::accept(KisProcessingVisitor &visitor, KisUndoAdapter *undoAdapter)
 {
-    return visitor.visit(this, undoAdapter);
+    visitor.visit(this, undoAdapter);
 }
 
 int KisNode::graphSequenceNumber() const
@@ -344,7 +326,7 @@ void KisNode::setParent(KisNodeWSP parent)
 KisNodeSP KisNode::parent() const
 {
     QReadLocker l(&m_d->nodeSubgraphLock);
-    return m_d->parent.isValid() ? KisNodeSP(m_d->parent) : 0;
+    return m_d->parent.isValid() ? KisNodeSP(m_d->parent) : KisNodeSP();
 }
 
 KisBaseNodeSP KisNode::parentCallback() const
@@ -367,6 +349,7 @@ void KisNode::baseNodeChangedCallback()
 {
     if(m_d->graphListener) {
         m_d->graphListener->nodeChanged(this);
+        emit sigNodeChangedInternal();
     }
 }
 
@@ -377,6 +360,12 @@ void KisNode::baseNodeInvalidateAllFramesCallback()
     }
 }
 
+void KisNode::addKeyframeChannel(KisKeyframeChannel *channel)
+{
+    channel->setNode(this);
+    KisBaseNode::addKeyframeChannel(channel);
+}
+
 KisNodeSP KisNode::firstChild() const
 {
     QReadLocker l(&m_d->nodeSubgraphLock);
@@ -505,13 +494,11 @@ KisNodeSP KisNode::findChildByName(const QString &name)
 
 bool KisNode::add(KisNodeSP newNode, KisNodeSP aboveThis)
 {
-    Q_ASSERT(newNode);
-
-    if (!newNode) return false;
-    if (aboveThis && aboveThis->parent().data() != this) return false;
-    if (!allowAsChild(newNode)) return false;
-    if (newNode->parent()) return false;
-    if (index(newNode) >= 0) return false;
+    KIS_SAFE_ASSERT_RECOVER_RETURN_VALUE(newNode, false);
+    KIS_SAFE_ASSERT_RECOVER_RETURN_VALUE(!aboveThis || aboveThis->parent().data() == this, false);
+    KIS_SAFE_ASSERT_RECOVER_RETURN_VALUE(allowAsChild(newNode), false);
+    KIS_SAFE_ASSERT_RECOVER_RETURN_VALUE(!newNode->parent(), false);
+    KIS_SAFE_ASSERT_RECOVER_RETURN_VALUE(index(newNode) < 0, false);
 
     int idx = aboveThis ? this->index(aboveThis) + 1 : 0;
 
@@ -539,6 +526,7 @@ bool KisNode::add(KisNodeSP newNode, KisNodeSP aboveThis)
         m_d->graphListener->nodeHasBeenAdded(this, idx);
     }
 
+    childNodeChanged(newNode);
 
     return true;
 }
@@ -566,6 +554,8 @@ bool KisNode::remove(quint32 index)
             m_d->graphListener->nodeHasBeenRemoved(this, index);
         }
 
+        childNodeChanged(removedNode);
+
         return true;
     }
     return false;
@@ -614,8 +604,8 @@ void KisNode::setDirty()
 
 void KisNode::setDirty(const QVector<QRect> &rects)
 {
-    Q_FOREACH (const QRect &rc, rects) {
-        setDirty(rc);
+    if(m_d->graphListener) {
+        m_d->graphListener->requestProjectionUpdate(this, rects, true);
     }
 }
 
@@ -625,9 +615,24 @@ void KisNode::setDirty(const QRegion &region)
 }
 
 void KisNode::setDirty(const QRect & rect)
+{
+    setDirty(QVector<QRect>({rect}));
+}
+
+void KisNode::setDirtyDontResetAnimationCache()
+{
+    setDirtyDontResetAnimationCache(QVector<QRect>({extent()}));
+}
+
+void KisNode::setDirtyDontResetAnimationCache(const QRect &rect)
+{
+    setDirtyDontResetAnimationCache(QVector<QRect>({rect}));
+}
+
+void KisNode::setDirtyDontResetAnimationCache(const QVector<QRect> &rects)
 {
     if(m_d->graphListener) {
-        m_d->graphListener->requestProjectionUpdate(this, rect);
+        m_d->graphListener->requestProjectionUpdate(this, rects, false);
     }
 }
 
