@@ -62,11 +62,12 @@ KisNodeProgressProxy::~KisNodeProgressProxy()
     delete d;
 }
 
-const KisNodeSP KisNodeProgressProxy::node() const
+void KisNodeProgressProxy::prepareDestroying()
 {
-    return d->node;
+    d->node = 0;
 }
 
+
 int KisNodeProgressProxy::maximum() const
 {
     return d->maximum;
