@@ -25,28 +25,28 @@
 
 #include <QPointer>
 #include <QPrinter>
+#include <QUuid>
+#include <QUrl>
 
 #include <xmlgui/kxmlguiwindow.h>
-#include <QUrl>
 #include <KoCanvasObserverBase.h>
 #include <KoCanvasSupervisor.h>
-
 #include "KisView.h"
 
 class QCloseEvent;
 class QMoveEvent;
 
 struct KoPageLayout;
-class KoCanvasResourceManager;
+class KoCanvasResourceProvider;
 
 class KisDocument;
-class KisView;
 class KisPrintJob;
 class KoDockFactoryBase;
 class QDockWidget;
 class KisView;
 class KisViewManager;
-
+class KoCanvasController;
+class KisWorkspaceResource;
 
 /**
  * @brief Main window for Krita
@@ -60,6 +60,15 @@ class KRITAUI_EXPORT KisMainWindow : public KXmlGuiWindow, public KoCanvasSuperv
 {
     Q_OBJECT
 
+public:
+    enum OpenFlag {
+        None = 0,
+        Import = 0x1,
+        BatchMode = 0x2,
+        RecoveryFile = 0x4
+    };
+    Q_DECLARE_FLAGS(OpenFlags, OpenFlag)
+
 public:
 
     /**
@@ -67,23 +76,14 @@ public:
      *
      *  Initializes a Calligra main window (with its basic GUI etc.).
      */
-    explicit KisMainWindow();
+    explicit KisMainWindow(QUuid id = QUuid());
 
     /**
      *  Destructor.
      */
-    virtual ~KisMainWindow();
+    ~KisMainWindow() override;
 
-    /**
-     * Update caption from document info - call when document info
-     * (title in the about page) changes.
-     */
-    void updateCaption();
-
-
-    // If noCleanup is set, KisMainWindow will not delete the root document
-    // or part manager on destruction.
-    void setNoCleanup(bool noCleanup);
+    QUuid id() const;
 
     /**
      * @brief showView shows the given view. Override this if you want to show
@@ -107,40 +107,98 @@ public:
      */
     void addRecentURL(const QUrl &url);
 
+    /**
+     * get list of URL strings for recent files
+     */
+    QList<QUrl> recentFilesUrls();
+    /**
+     * removes the given url from the list of recent files
+     */
+    void removeRecentUrl(const QUrl &url);
+
+
     /**
      * Load the desired document and show it.
      * @param url the URL to open
      *
      * @return TRUE on success.
      */
-    bool openDocument(const QUrl &url);
+    bool openDocument(const QUrl &url, OpenFlags flags);
+
+    /**
+     * Activate a view containing the document in this window, creating one if needed.
+     */
+    void showDocument(KisDocument *document);
+
+
+    /**
+     * Toggles between showing the welcome screen and the MDI area
+     *
+     *  hack: There seems to be a bug that prevents events happening to the MDI area if it
+     *  isn't actively displayed (set in the widgetStack). This can cause things like the title bar
+     *  not to update correctly Before doing any actions related to opening or creating documents,
+     *  make sure to switch this first to make sure everything can communicate to the MDI area correctly
+     */
+    void showWelcomeScreen(bool show);
+
+    /**
+     * Saves the document, asking for a filename if necessary.
+     *
+     * @param saveas if set to TRUE the user is always prompted for a filename
+     * @param silent if set to TRUE rootDocument()->setTitleModified will not be called.
+     *
+     * @return TRUE on success, false on error or cancel
+     *         (don't display anything in this case, the error dialog box is also implemented here
+     *         but restore the original URL in slotFileSaveAs)
+     */
+    bool saveDocument(KisDocument *document, bool saveas, bool isExporting);
 
     void setReadWrite(bool readwrite);
 
-
     /// Return the list of dock widgets belonging to this main window.
     QList<QDockWidget*> dockWidgets() const;
 
     QDockWidget* dockWidget(const QString &id);
 
-    QList<KoCanvasObserverBase*> canvasObservers() const;
+    QList<KoCanvasObserverBase*> canvasObservers() const override;
 
-    KoCanvasResourceManager *resourceManager() const;
+    KoCanvasResourceProvider *resourceManager() const;
 
     int viewCount() const;
 
+    void saveWindowState(bool restoreNormalState =false);
+
+    const KConfigGroup &windowStateConfig() const;
+
     /**
      * A wrapper around restoreState
      * @param state the saved state
      * @return TRUE on success
      */
-    bool restoreWorkspace(const QByteArray &state);
+    bool restoreWorkspace(KisWorkspaceResource *workspace);
+    bool restoreWorkspaceState(const QByteArray &state);
+
+    static void swapWorkspaces(KisMainWindow *a, KisMainWindow *b);
 
     KisViewManager *viewManager() const;
 
-    void addViewAndNotifyLoadingCompleted(KisDocument *document);
+    KisView *addViewAndNotifyLoadingCompleted(KisDocument *document);
 
-    QStringList showOpenFileDialog();
+    QStringList showOpenFileDialog(bool isImporting);
+
+    /**
+     * Shows if the main window is saving anything right now. If the
+     * user presses Ctrl+W too fast, then the document can be close
+     * before the saving is completed. I'm not sure if it is fixable
+     * in any way without avoiding using porcessEvents()
+     * everywhere (DK)
+     *
+     * Don't use it unless you have no option.
+     */
+    bool hackIsSaving() const;
+
+    /// Copy the given file into the bundle directory.
+    bool installBundle(const QString &fileName) const;
 
 Q_SIGNALS:
 
@@ -166,11 +224,18 @@ Q_SIGNALS:
 
     void guiLoadingFinished();
 
-    /// This signal is emitted when the user clicked on the progressbar cancel
-    void sigProgressCanceled();
+    /// emitted when the window is migrated among different screens
+    void screenChanged();
 
 public Q_SLOTS:
 
+
+    /**
+     * clears the list of the recent files
+     */
+    void clearRecentFiles();
+
+
     /**
      *  Slot for opening a new document.
      *
@@ -185,7 +250,7 @@ public Q_SLOTS:
      *  If the current document is empty, the opened document replaces it.
      *  If not a new mainwindow will be opened for showing the opened file.
      */
-    void slotFileOpen();
+    void slotFileOpen(bool isImporting = false);
 
     /**
      *  Slot for opening a file among the recently opened files.
@@ -200,34 +265,44 @@ public Q_SLOTS:
      */
     void slotPreferences();
 
+    /**
+     * Update caption from document info - call when document info
+     * (title in the about page) changes.
+     */
+    void updateCaption();
+
     /**
      *  Saves the current document with the current name.
      */
     void slotFileSave();
 
-    KisPrintJob* exportToPdf(const QString &pdfFileName = QString());
 
-    void slotProgress(int value);
+    void slotShowSessionManager();
 
-    /**
-     * Saves the document, asking for a filename if necessary.
-     *
-     * @param saveas if set to TRUE the user is always prompted for a filename
-     *
-     * @param silent if set to TRUE rootDocument()->setTitleModified will not be called.
-     *
-     * @param specialOutputFlag set to enums defined in KisDocument if save to special output format
-     *
-     * @return TRUE on success, false on error or cancel
-     *         (don't display anything in this case, the error dialog box is also implemented here
-     *         but restore the original URL in slotFileSaveAs)
-     */
-    bool saveDocument(KisDocument *document, bool saveas = false, bool silent = false, int specialOutputFlag = 0);
+    // XXX: disabled
+    KisPrintJob* exportToPdf(QString pdfFileName = QString());
 
     /**
      * Update the option widgets to the argument ones, removing the currently set widgets.
      */
-    void newOptionWidgets(const QList<QPointer<QWidget> > & optionWidgetList);
+    void newOptionWidgets(KoCanvasController *controller, const QList<QPointer<QWidget> > & optionWidgetList);
+
+    KisView *newView(QObject *document);
+
+    void notifyChildViewDestroyed(KisView *view);
+
+    /// Set the active view, this will update the undo/redo actions
+    void setActiveView(KisView *view);
+
+    void subWindowActivated();
+
+    void windowFocused();
+
+    /**
+     * Reloads the recent documents list.
+     */
+    void reloadRecentFileList();
+
 
 
 private Q_SLOTS:
@@ -242,12 +317,10 @@ private Q_SLOTS:
     void slotSaveCanceled(const QString &);
     void forceDockTabFonts();
 
-    void slotProgressCanceled();
-
     /**
      * @internal
      */
-    void slotDocumentTitleModified(const QString &caption, bool mod);
+    void slotDocumentTitleModified();
 
     /**
      *  Prints the actual document.
@@ -261,10 +334,7 @@ private Q_SLOTS:
 
     void slotFilePrintPreview();
 
-    KisPrintJob* exportToPdf(KoPageLayout pageLayout, QString pdfFileName = QString());
-
     void importAnimation();
-    void exportAnimation();
 
     /**
      * Show a dialog with author and document information.
@@ -307,29 +377,23 @@ private Q_SLOTS:
      */
     void viewFullscreen(bool fullScreen);
 
-    /**
-     * Toggle docker titlebars on/off.
-     */
-    void showDockerTitleBars(bool show);
-
     /**
      * Reload file
      */
     void slotReloadFile();
 
+
     /**
      * File --> Import
      *
-     * This will call slotFileOpen().  To differentiate this from an ordinary
-     * call to slotFileOpen() call @ref isImporting().
+     * This will call slotFileOpen().
      */
     void slotImportFile();
 
     /**
      * File --> Export
      *
-     * This will call slotFileSaveAs().  To differentiate this from an ordinary
-     * call to slotFileSaveAs() call @ref isExporting().
+     * This will call slotFileSaveAs().
      */
     void slotExportFile();
 
@@ -345,34 +409,40 @@ private Q_SLOTS:
 
     void undo();
     void redo();
-    void subWindowActivated();
     void updateWindowMenu();
     void setActiveSubWindow(QWidget *window);
     void configChanged();
-    void newView(QObject *document);
+
     void newWindow();
     void closeCurrentWindow();
     void checkSanity();
+
     /// Quits Krita with error message from m_errorMessage.
     void showErrorAndDie();
 
+    void initializeGeometry();
+    void showManual();
+    void switchTab(int index);
+
+    void windowScreenChanged(QScreen *screen);
+
+    void slotXmlGuiMakingChanges(bool finished);
+
 protected:
 
-    void closeEvent(QCloseEvent * e);
-    void resizeEvent(QResizeEvent * e);
-
-    /// Set the active view, this will update the undo/redo actions
-    virtual void setActiveView(KisView *view);
+    void closeEvent(QCloseEvent * e) override;
+    void resizeEvent(QResizeEvent * e) override;
 
     // QWidget overrides
-    virtual void dragEnterEvent(QDragEnterEvent * event);
-    virtual void dropEvent(QDropEvent * event);
-    virtual void dragMoveEvent(QDragMoveEvent * event);
-    virtual void dragLeaveEvent(QDragLeaveEvent * event);
-
-    void setToolbarList(QList<QAction*> toolbarList);
 
 private:
+
+    friend class KisWelcomePageWidget;
+    void dragMove(QDragMoveEvent *event);
+    void dragLeave();
+
+private:
+
     /**
      * Add a the given view to the list of views of this mainwindow.
      * This is a private implementation. For public usage please use
@@ -380,8 +450,7 @@ private:
      */
     void addView(KisView *view);
 
-
-    friend class KisApplication;
+    friend class KisPart;
 
 
     /**
@@ -392,63 +461,30 @@ private:
      */
     QDockWidget* createDockWidget(KoDockFactoryBase* factory);
 
-    bool openDocumentInternal(const QUrl &url, KisDocument *newdoc = 0);
+    bool openDocumentInternal(const QUrl &url, KisMainWindow::OpenFlags flags = 0);
 
-    /**
-     * Returns whether or not the current slotFileSave[As]() or saveDocument()
-     * call is actually an export operation (like File --> Export).
-     *
-     * If this is true, you must call KisDocument::export() instead of
-     * KisDocument::save() or KisDocument::saveAs(), in any reimplementation of
-     * saveDocument().
-     */
-    bool isExporting() const;
-
-    /**
-     * Returns whether or not the current slotFileOpen() or openDocument()
-     * call is actually an import operation (like File --> Import).
-     *
-     * If this is true, you must call KisDocument::import() instead of
-     * KisDocument::openUrl(), in any reimplementation of openDocument() or
-     * openDocumentInternal().
-     */
-    bool isImporting() const;
-
-    /**
-     * Reloads the recent documents list.
-     */
-    void reloadRecentFileList();
 
     /**
      * Updates the window caption based on the document info and path.
      */
-    void updateCaption(const QString & caption, bool mod);
+    void updateCaption(const QString & caption, bool modified);
     void updateReloadFileAction(KisDocument *doc);
 
     void saveWindowSettings();
 
-    QPointer<KisView>activeKisView();
+    QPointer<KisView> activeKisView();
 
     void applyDefaultSettings(QPrinter &printer);
 
-    bool exportConfirmation(const QByteArray &outputFormat);
-
     void createActions();
 
     void applyToolBarLayout();
 
-protected:
+    QByteArray borrowWorkspace(KisMainWindow *borrower);
 
-    void moveEvent(QMoveEvent *e);
-
-private Q_SLOTS:
-    void initializeGeometry();
-    void showManual();
-    void switchTab(int index);
 
 private:
 
-
     /**
      * Struct used in the list created by createCustomDocumentWidgets()
      */
@@ -469,4 +505,6 @@ private:
 
 };
 
+Q_DECLARE_OPERATORS_FOR_FLAGS(KisMainWindow::OpenFlags)
+
 #endif
