@@ -24,6 +24,7 @@
 #include <QMimeData>
 #include <QPixmap>
 #include <QImage>
+#include <QThread>
 
 #include "kis_action_manager.h"
 #include "kactioncollection.h"
@@ -44,15 +45,59 @@
 #include <KisPart.h>
 
 #include <kritaversion.h>
+#include <QSysInfo>
+#include <kis_config.h>
+#include <kis_image_config.h>
+#include "opengl/kis_opengl.h"
+
+
+
 
 KisWelcomePageWidget::KisWelcomePageWidget(QWidget *parent)
     : QWidget(parent)
 {
     setupUi(this);
 
-#ifndef KRITA_BETA
-    lblSurveyLink->setVisible(false);
+#ifdef KRITA_BETA
+    lblSurveyLink->setVisible(true);
     lblSurveyLink->setTextInteractionFlags(Qt::TextBrowserInteraction);
+
+    QString surveyId = "965214";
+
+    QString arguments;
+    arguments += "&KritaVersion=" + QStringLiteral(KRITA_VERSION_STRING);
+    arguments += "&BuildAbi=" + QSysInfo::buildAbi();
+    arguments += "&BuildCPUArchitecture=" + QSysInfo::buildCpuArchitecture();
+    arguments += "&KernelVersion=" + QSysInfo::kernelVersion();
+    arguments += "&ProductType=" + QSysInfo::productType();
+    arguments += "&ProductVersion=" + QSysInfo::productVersion();
+    arguments += "&InstantPreviewEnable=" + QString(KisConfig(true).levelOfDetailEnabled( )? "true" : "false");
+    arguments += "&TotalMemorySize=" + QString::number(KisImageConfig(true).totalRAM());
+    arguments += "&SwapLocation=" + KisImageConfig(true).swapDir();
+    arguments += "&NumberOfCores=" + QString::number(QThread::idealThreadCount());
+    arguments += "&HiDPI=" + QString(QCoreApplication::testAttribute(Qt::AA_EnableHighDpiScaling) ? "true" : "false");
+    arguments += "&OpenGLRenderer=" + KisOpenGL::convertOpenGLRendererToConfig(KisOpenGL::getCurrentOpenGLRenderer());
+
+    QString openGLInfo = KisOpenGL::getDebugText();
+    QStringList infolines = openGLInfo.split("\n");
+    Q_FOREACH(const QString line, infolines) {
+        if (line.contains("Vendor")) {
+            arguments += "&GPUVendor=" + line.split(": ")[1].replace("\n", "");
+        }
+        if (line.contains("Version")) {
+            arguments += "&OpenGLVersion=" + line.split(": ")[1].replace("\n", "");
+        }
+    }
+    QUrl url (arguments);
+
+    QString source = QString("<html><head/><body><p align=\"center\"><a href=\"https://survey.kde.org/index.php/%1?lang=en?%2\">"
+                             "<span style=\"font-size:18pt; font-weight:600; text-decoration: underline; color:#2980b9;\">"
+                             "Help Krita, do the survey!"
+                              "</span></a></p></body></html>").arg(surveyId).arg(QString::fromUtf8(url.toEncoded()));
+
+    lblSurveyLink->setText(source);
+#else
+    lblSurveyLink->setVisible(false);
 #endif
 
     recentDocumentsListView->setDragEnabled(false);
@@ -125,7 +170,7 @@ void KisWelcomePageWidget::setMainWindow(KisMainWindow* mainWin)
         // until after the view manager is set
         connect(newFileLink, SIGNAL(clicked(bool)), this, SLOT(slotNewFileClicked()));
         connect(openFileLink, SIGNAL(clicked(bool)), this, SLOT(slotOpenFileClicked()));
-        connect(clearRecentFilesLink, SIGNAL(clicked(bool)), this, SLOT(slotClearRecentFiles()));
+        connect(clearRecentFilesLink, SIGNAL(clicked(bool)), mainWin, SLOT(clearRecentFiles()));
 
         slotUpdateThemeColors();
 
@@ -218,7 +263,6 @@ void KisWelcomePageWidget::slotUpdateThemeColors()
 
     poweredByKDELink->setText(QString("<a style=\"color: " + blendedColor.name() + " \" href=\"https://userbase.kde.org/What_is_KDE?" + analyticsString + "what-is-kde" + "\">")
                               .append(i18n("Powered by KDE")).append("</a>"));
-
     // re-populate recent files since they might have themed icons
     populateRecentDocuments();
 
@@ -239,6 +283,8 @@ void KisWelcomePageWidget::populateRecentDocuments()
         QString recentFileUrlPath = m_mainWindow->recentFilesUrls().at(i).toLocalFile();
         QString fileName = recentFileUrlPath.split("/").last();
 
+        QList<QUrl> brokenUrls;
+
         if (m_thumbnailMap.contains(recentFileUrlPath)) {
             recentItem->setIcon(m_thumbnailMap[recentFileUrlPath]);
         }
@@ -268,16 +314,23 @@ void KisWelcomePageWidget::populateRecentDocuments()
                             }
                         }
                     }
+                    else {
+                        brokenUrls << m_mainWindow->recentFilesUrls().at(i);
+                    }
                 }
                 else if (fi.suffix() == "tiff" || fi.suffix() == "tif") {
                     // Workaround for a bug in Qt tiff QImageIO plugin
                     QScopedPointer<KisDocument> doc;
                     doc.reset(KisPart::instance()->createDocument());
+                    doc->setFileBatchMode(true);
                     bool r = doc->openUrl(QUrl::fromLocalFile(recentFileUrlPath), KisDocument::DontAddToRecent);
                     if (r) {
                         KisPaintDeviceSP projection = doc->image()->projection();
                         recentItem->setIcon(QIcon(QPixmap::fromImage(projection->createThumbnail(48, 48, projection->exactBounds()))));
                     }
+                    else {
+                        brokenUrls << m_mainWindow->recentFilesUrls().at(i);
+                    }
                 }
                 else {
                     QImage img;
@@ -286,15 +339,24 @@ void KisWelcomePageWidget::populateRecentDocuments()
                     if (!img.isNull()) {
                         recentItem->setIcon(QIcon(QPixmap::fromImage(img.scaledToWidth(48))));
                     }
+                    else {
+                        brokenUrls << m_mainWindow->recentFilesUrls().at(i);
+                    }
+                }
+                if (brokenUrls.size() > 0 && brokenUrls.last().toLocalFile() != recentFileUrlPath) {
+                    m_thumbnailMap[recentFileUrlPath] = recentItem->icon();
                 }
-                m_thumbnailMap[recentFileUrlPath] = recentItem->icon();
             }
         }
-
+        Q_FOREACH(const QUrl &url, brokenUrls) {
+            m_mainWindow->removeRecentUrl(url);
+        }
         // set the recent object with the data
-        recentItem->setText(fileName); // what to display for the item
-        recentItem->setToolTip(recentFileUrlPath);
-        m_recentFilesModel.appendRow(recentItem);
+        if (brokenUrls.isEmpty() || brokenUrls.last().toLocalFile() != recentFileUrlPath) {
+            recentItem->setText(fileName); // what to display for the item
+            recentItem->setToolTip(recentFileUrlPath);
+            m_recentFilesModel.appendRow(recentItem);
+        }
     }
 
     // hide clear and Recent files title if there are none
@@ -377,8 +439,3 @@ void KisWelcomePageWidget::slotOpenFileClicked()
     m_mainWindow->slotFileOpen();
 }
 
-void KisWelcomePageWidget::slotClearRecentFiles()
-{
-    m_mainWindow->clearRecentFiles();
-    populateRecentDocuments();
-}
