@@ -31,7 +31,7 @@
 #include <kis_config.h>
 
 #include "kis_global.h"
-#include <squeezedcombobox.h>
+#include <KisSqueezedComboBox.h>
 
 #include "ui_wdggeneralsettings.h"
 #include "ui_wdgdisplaysettings.h"
@@ -80,16 +80,17 @@ public:
     void setDefault();
     int undoStackSize();
     bool showOutlineWhilePainting();
-    bool hideSplashScreen();
 
     int mdiMode();
     int favoritePresets();
     bool showCanvasMessages();
     bool compressKra();
+    bool useZip64();
     bool toolOptionsInDocker();
+    bool kineticScrollingEnabled();
     int kineticScrollingGesture();
     int kineticScrollingSensitivity();
-    bool kineticScrollingScrollbar();
+    bool kineticScrollingHiddenScrollbars();
     bool switchSelectionCtrlAlt();
     bool convertToImageColorspaceOnImport();
 
@@ -173,7 +174,7 @@ public:
     WdgColorSettings  *m_page;
     QButtonGroup m_pasteBehaviourGroup;
     QList<QLabel*> m_monitorProfileLabels;
-    QList<SqueezedComboBox*> m_monitorProfileWidgets;
+    QList<KisSqueezedComboBox*> m_monitorProfileWidgets;
 };
 
 //=======================
@@ -196,6 +197,7 @@ public:
 
 private Q_SLOTS:
     void slotTabletTest();
+    void slotResolutionSettings();
 
 public:
     void setDefault();
@@ -278,6 +280,7 @@ public:
     void setDefault();
 protected Q_SLOTS:
     void slotUseOpenGLToggled(bool isChecked);
+    void slotPreferredSurfaceFormatChanged(int index);
 
 public:
 };
