@@ -111,6 +111,16 @@ KoUnit KisShapeLayerCanvasBase::unit() const
     return KoUnit(KoUnit::Point);
 }
 
+void KisShapeLayerCanvasBase::setUpdatesBlocked(bool value)
+{
+    m_updatesBlocked = value;
+}
+
+bool KisShapeLayerCanvasBase::updatesBlocked() const
+{
+    return m_updatesBlocked;
+}
+
 void KisShapeLayerCanvasBase::prepareForDestroying()
 {
     m_isDestroying = true;
@@ -126,9 +136,8 @@ KisShapeLayerCanvas::KisShapeLayerCanvas(KisShapeLayer *parent, KisImageWSP imag
         : KisShapeLayerCanvasBase(parent, image)
         , m_projection(0)
         , m_parentLayer(parent)
-        , m_canvasUpdateCompressor(new KisSignalCompressor(500, KisSignalCompressor::FIRST_INACTIVE, this))
+        , m_canvasUpdateCompressor(100, KisSignalCompressor::FIRST_INACTIVE)
         , m_asyncUpdateSignalCompressor(100, KisSignalCompressor::FIRST_INACTIVE)
-        , m_image(image)
 {
     /**
      * The layour should also add itself to its own shape manager, so that the canvas
@@ -137,13 +146,11 @@ KisShapeLayerCanvas::KisShapeLayerCanvas(KisShapeLayer *parent, KisImageWSP imag
     m_shapeManager->addShape(parent, KoShapeManager::AddWithoutRepaint);
     m_shapeManager->selection()->setActiveLayer(parent);
 
-    connect(this, SIGNAL(forwardRepaint()), m_canvasUpdateCompressor, SLOT(start()));
-    connect(m_canvasUpdateCompressor, SIGNAL(timeout()), this, SLOT(repaint()));
-
     connect(&m_asyncUpdateSignalCompressor, SIGNAL(timeout()), SLOT(slotStartAsyncRepaint()));
+    connect(this, SIGNAL(forwardRepaint()), &m_canvasUpdateCompressor, SLOT(start()));
+    connect(&m_canvasUpdateCompressor, SIGNAL(timeout()), this, SLOT(slotStartDirectSyncRepaint()));
 
-    connect(m_image, SIGNAL(sigSizeChanged(const QPointF &, const QPointF &)), SLOT(slotImageSizeChanged()));
-    m_cachedImageRect = m_image->bounds();
+    setImage(image);
 }
 
 KisShapeLayerCanvas::~KisShapeLayerCanvas()
@@ -153,7 +160,19 @@ KisShapeLayerCanvas::~KisShapeLayerCanvas()
 
 void KisShapeLayerCanvas::setImage(KisImageWSP image)
 {
+    if (m_image) {
+        disconnect(m_image, 0, this, 0);
+    }
+
     m_viewConverter->setImage(image);
+    m_image = image;
+
+    if (image) {
+        connect(m_image, SIGNAL(sigSizeChanged(QPointF,QPointF)), SLOT(slotImageSizeChanged()));
+        m_cachedImageRect = m_image->bounds();
+    }
+
+    updateUpdateCompressorDelay();
 }
 
 
@@ -198,7 +217,7 @@ private:
 
 void KisShapeLayerCanvas::updateCanvas(const QVector<QRectF> &region)
 {
-    if (!m_parentLayer->image() || m_isDestroying) {
+    if (!m_parentLayer->image() || m_isDestroying || m_updatesBlocked) {
         return;
     }
 
@@ -231,6 +250,7 @@ void KisShapeLayerCanvas::updateCanvas(const QVector<QRectF> &region)
 
     if (qApp->thread() == QThread::currentThread()) {
         emit forwardRepaint();
+        m_hasDirectSyncRepaintInitiated = true;
     } else {
         m_asyncUpdateSignalCompressor.start();
         m_hasUpdateInCompressor = true;
@@ -249,6 +269,12 @@ void KisShapeLayerCanvas::slotStartAsyncRepaint()
     m_image->addSpontaneousJob(new KisRepaintShapeLayerLayerJob(m_parentLayer, this));
 }
 
+void KisShapeLayerCanvas::slotStartDirectSyncRepaint()
+{
+    m_hasDirectSyncRepaintInitiated = false;
+    repaint();
+}
+
 void KisShapeLayerCanvas::slotImageSizeChanged()
 {
     QRegion dirtyCacheRegion;
@@ -263,25 +289,35 @@ void KisShapeLayerCanvas::slotImageSizeChanged()
     updateCanvas(dirtyRects);
 
     m_cachedImageRect = m_image->bounds();
+    updateUpdateCompressorDelay();
 }
 
 void KisShapeLayerCanvas::repaint()
 {
     QRect repaintRect;
+    bool forceUpdateHiddenAreasOnly = false;
 
     {
         QMutexLocker locker(&m_dirtyRegionMutex);
         repaintRect = m_dirtyRegion.boundingRect();
+        forceUpdateHiddenAreasOnly = m_forceUpdateHiddenAreasOnly;
+
         m_dirtyRegion = QRegion();
+        m_forceUpdateHiddenAreasOnly = false;
     }
 
-    if (repaintRect.isEmpty()) {
-        return;
-    }
+    if (!forceUpdateHiddenAreasOnly) {
+        if (repaintRect.isEmpty()) {
+            return;
+        }
 
-    // Crop the update rect by the image bounds. We keep the cache consistent
-    // by tracking the size of the image in slotImageSizeChanged()
-    repaintRect = repaintRect.intersected(m_parentLayer->image()->bounds());
+        // Crop the update rect by the image bounds. We keep the cache consistent
+        // by tracking the size of the image in slotImageSizeChanged()
+        repaintRect = repaintRect.intersected(m_parentLayer->image()->bounds());
+    } else {
+        const QRectF shapesBounds = KoShape::boundingRect(m_shapeManager->shapes());
+        repaintRect = kisGrowRect(m_viewConverter->documentToView(shapesBounds).toAlignedRect(), 2);
+    }
 
     QImage image(repaintRect.width(), repaintRect.height(), QImage::Format_ARGB32);
     image.fill(0);
@@ -302,8 +338,14 @@ void KisShapeLayerCanvas::repaint()
     KisPaintDeviceSP dev = new KisPaintDevice(m_projection->colorSpace());
     dev->convertFromQImage(image, 0);
 
+    if (forceUpdateHiddenAreasOnly) {
+        m_projection->clear();
+    }
+
     KisPainter::copyAreaOptimized(repaintRect.topLeft(), dev, m_projection, QRect(QPoint(), repaintRect.size()));
 
+    m_projection->purgeDefaultPixels();
+
     m_parentLayer->setDirty(repaintRect);
 
     m_hasChangedWhileBeingInvisible |= !m_parentLayer->visible(true);
@@ -320,12 +362,32 @@ void KisShapeLayerCanvas::forceRepaint()
      * The only real solution to this is to port vector tools to strokes framework.
      */
 
-    if (m_hasUpdateInCompressor) {
+    if (hasPendingUpdates()) {
         m_asyncUpdateSignalCompressor.stop();
         slotStartAsyncRepaint();
     }
 }
 
+bool KisShapeLayerCanvas::hasPendingUpdates() const
+{
+    return m_hasUpdateInCompressor || m_hasDirectSyncRepaintInitiated;
+}
+
+void KisShapeLayerCanvas::forceRepaintWithHiddenAreas()
+{
+    KIS_SAFE_ASSERT_RECOVER_RETURN(m_parentLayer->image());
+    KIS_SAFE_ASSERT_RECOVER_RETURN(!m_isDestroying);
+    KIS_SAFE_ASSERT_RECOVER_RETURN(!m_updatesBlocked);
+
+    {
+        QMutexLocker locker(&m_dirtyRegionMutex);
+        m_forceUpdateHiddenAreasOnly = true;
+    }
+
+    m_asyncUpdateSignalCompressor.stop();
+    slotStartAsyncRepaint();
+}
+
 void KisShapeLayerCanvas::resetCache()
 {
     m_projection->clear();
@@ -343,3 +405,14 @@ void KisShapeLayerCanvas::rerenderAfterBeingInvisible()
     m_hasChangedWhileBeingInvisible = false;
     resetCache();
 }
+
+void KisShapeLayerCanvas::updateUpdateCompressorDelay()
+{
+    if (m_cachedImageRect.width() * m_cachedImageRect.height() < 2480 * 3508) { // A4 300 DPI
+        m_canvasUpdateCompressor.setDelay(25);
+    } else if (m_cachedImageRect.width() * m_cachedImageRect.height() < 4961 * 7061) { // A4 600 DPI
+        m_canvasUpdateCompressor.setDelay(100);
+    } else { // Really big
+        m_canvasUpdateCompressor.setDelay(500);
+    }
+}
