@@ -25,7 +25,7 @@
 #include <QDialog>
 #include <QVBoxLayout>
 #include <QFileInfo>
-#include <QDesktopServices>
+#include <QStandardPaths>
 
 #include <kactioncollection.h>
 #include <klocalizedstring.h>
@@ -60,12 +60,11 @@
 #include <flake/kis_shape_layer.h>
 #include <kis_undo_adapter.h>
 #include <kis_painter.h>
-#include <metadata/kis_meta_data_store.h>
-#include <metadata/kis_meta_data_merge_strategy_registry.h>
+#include <kis_meta_data_store.h>
+#include <kis_meta_data_merge_strategy_registry.h>
 #include <kis_psd_layer_style.h>
 #include <KisMimeDatabase.h>
 
-#include "KisImportExportManager.h"
 #include "kis_config.h"
 #include "kis_cursor.h"
 #include "dialogs/kis_dlg_adj_layer_props.h"
@@ -74,13 +73,13 @@
 #include "dialogs/kis_dlg_generator_layer.h"
 #include "dialogs/kis_dlg_file_layer.h"
 #include "dialogs/kis_dlg_layer_style.h"
-#include "KisDocument.h"
+#include "dialogs/KisDlgChangeCloneSource.h"
 #include "kis_filter_manager.h"
 #include "kis_node_visitor.h"
 #include "kis_paint_layer.h"
 #include "commands/kis_image_commands.h"
-#include "commands/kis_layer_command.h"
 #include "commands/kis_node_commands.h"
+#include "kis_change_file_layer_command.h"
 #include "kis_canvas_resource_provider.h"
 #include "kis_selection_manager.h"
 #include "kis_statusbar.h"
@@ -94,9 +93,7 @@
 #include "kis_node_manager.h"
 #include "kis_action.h"
 #include "kis_action_manager.h"
-#include "KisPart.h"
 #include "kis_raster_keyframe_channel.h"
-
 #include "kis_signal_compressor_with_param.h"
 #include "kis_abstract_projection_plane.h"
 #include "commands_new/kis_set_layer_style_command.h"
@@ -104,6 +101,7 @@
 #include "kis_selection_mask.h"
 #include "kis_layer_utils.h"
 #include "lazybrush/kis_colorize_mask.h"
+#include "kis_processing_applicator.h"
 
 #include "KisSaveGroupVisitor.h"
 
@@ -153,7 +151,7 @@ void KisLayerManager::activateLayer(KisLayerSP layer)
         emit sigLayerActivated(layer);
         layersUpdated();
         if (layer) {
-            m_view->resourceProvider()->slotNodeActivated(layer.data());
+            m_view->canvasResourceProvider()->slotNodeActivated(layer.data());
         }
     }
 }
@@ -192,20 +190,16 @@ void KisLayerManager::setup(KisActionManager* actionManager)
 
 void KisLayerManager::updateGUI()
 {
-    KisImageWSP image = m_view->image();
+    KisImageSP image = m_view->image();
+    KisLayerSP layer = activeLayer();
 
-    KisLayerSP layer;
-    qint32 nlayers = 0;
+    const bool isGroupLayer = layer && layer->inherits("KisGroupLayer");
 
-    if (image) {
-        layer = activeLayer();
-        nlayers = image->nlayers();
-    }
-
-    // XXX these should be named layer instead of image
-    m_imageFlatten->setEnabled(nlayers > 1);
-    m_imageMergeLayer->setEnabled(nlayers > 1 && layer && layer->prevSibling());
-    m_flattenLayer->setEnabled(nlayers > 1 && layer && layer->firstChild());
+    m_imageMergeLayer->setText(
+                isGroupLayer ?
+                    i18nc("@action:inmenu", "Merge Group") :
+                    i18nc("@action:inmenu", "Merge with Layer Below"));
+    m_flattenLayer->setVisible(!isGroupLayer);
 
     if (m_view->statusBar())
         m_view->statusBar()->setProfile(image);
@@ -222,9 +216,9 @@ void KisLayerManager::imageResizeToActiveLayer()
             image->cropImage(cropRect);
         } else {
             m_view->showFloatingMessage(
-                i18nc("floating message in layer manager",
-                      "Layer is empty "),
-                QIcon(), 2000, KisFloatingMessage::Low);
+                        i18nc("floating message in layer manager",
+                              "Layer is empty "),
+                        QIcon(), 2000, KisFloatingMessage::Low);
         }
     }
 }
@@ -243,30 +237,31 @@ void KisLayerManager::layerProperties()
     if (!m_view->document()) return;
 
     KisLayerSP layer = activeLayer();
+    if (!layer) return;
+
     QList<KisNodeSP> selectedNodes = m_view->nodeManager()->selectedNodes();
     const bool multipleLayersSelected = selectedNodes.size() > 1;
 
-    if (!layer) return;
+    KisAdjustmentLayerSP adjustmentLayer = KisAdjustmentLayerSP(dynamic_cast<KisAdjustmentLayer*>(layer.data()));
+    KisGeneratorLayerSP groupLayer = KisGeneratorLayerSP(dynamic_cast<KisGeneratorLayer*>(layer.data()));
+    KisFileLayerSP filterLayer = KisFileLayerSP(dynamic_cast<KisFileLayer*>(layer.data()));
 
-    KisAdjustmentLayerSP alayer = KisAdjustmentLayerSP(dynamic_cast<KisAdjustmentLayer*>(layer.data()));
-    KisGeneratorLayerSP glayer = KisGeneratorLayerSP(dynamic_cast<KisGeneratorLayer*>(layer.data()));
+    if (adjustmentLayer && !multipleLayersSelected) {
 
-    if (alayer && !multipleLayersSelected) {
+        KisPaintDeviceSP dev = adjustmentLayer->projection();
 
-        KisPaintDeviceSP dev = alayer->projection();
-
-        KisDlgAdjLayerProps dlg(alayer, alayer.data(), dev, m_view, alayer->filter().data(), alayer->name(), i18n("Filter Layer Properties"), m_view->mainWindow(), "dlgadjlayerprops");
+        KisDlgAdjLayerProps dlg(adjustmentLayer, adjustmentLayer.data(), dev, m_view, adjustmentLayer->filter().data(), adjustmentLayer->name(), i18n("Filter Layer Properties"), m_view->mainWindow(), "dlgadjlayerprops");
         dlg.resize(dlg.minimumSizeHint());
 
 
-        KisFilterConfigurationSP configBefore(alayer->filter());
+        KisFilterConfigurationSP configBefore(adjustmentLayer->filter());
         KIS_ASSERT_RECOVER_RETURN(configBefore);
         QString xmlBefore = configBefore->toXML();
 
 
         if (dlg.exec() == QDialog::Accepted) {
 
-            alayer->setName(dlg.layerName());
+            adjustmentLayer->setName(dlg.layerName());
 
             KisFilterConfigurationSP configAfter(dlg.filterConfiguration());
             Q_ASSERT(configAfter);
@@ -274,12 +269,12 @@ void KisLayerManager::layerProperties()
 
             if(xmlBefore != xmlAfter) {
                 KisChangeFilterCmd *cmd
-                   = new KisChangeFilterCmd(alayer,
-                                             configBefore->name(),
-                                             xmlBefore,
-                                             configAfter->name(),
-                                             xmlAfter,
-                                             false);
+                        = new KisChangeFilterCmd(adjustmentLayer,
+                                                 configBefore->name(),
+                                                 xmlBefore,
+                                                 configAfter->name(),
+                                                 xmlAfter,
+                                                 false);
                 // FIXME: check whether is needed
                 cmd->redo();
                 m_view->undoAdapter()->addCommand(cmd);
@@ -292,45 +287,58 @@ void KisLayerManager::layerProperties()
             QString xmlAfter = configAfter->toXML();
 
             if(xmlBefore != xmlAfter) {
-                alayer->setFilter(KisFilterRegistry::instance()->cloneConfiguration(configBefore.data()));
-                alayer->setDirty();
+                adjustmentLayer->setFilter(KisFilterRegistry::instance()->cloneConfiguration(configBefore.data()));
+                adjustmentLayer->setDirty();
             }
         }
     }
-    else if (glayer && !multipleLayersSelected) {
-
-        KisDlgGeneratorLayer dlg(glayer->name(), m_view, m_view->mainWindow());
-        dlg.setCaption(i18n("Fill Layer Properties"));
-
-        KisFilterConfigurationSP configBefore(glayer->filter());
+    else if (groupLayer && !multipleLayersSelected) {
+        KisFilterConfigurationSP configBefore(groupLayer->filter());
         Q_ASSERT(configBefore);
         QString xmlBefore = configBefore->toXML();
 
-        dlg.setConfiguration(configBefore.data());
-        dlg.resize(dlg.minimumSizeHint());
+        KisDlgGeneratorLayer *dlg = new KisDlgGeneratorLayer(groupLayer->name(), m_view, m_view->mainWindow(), groupLayer, configBefore);
+        dlg->setCaption(i18n("Fill Layer Properties"));
+        dlg->setAttribute(Qt::WA_DeleteOnClose);
+
+        dlg->setConfiguration(configBefore.data());
+        dlg->resize(dlg->minimumSizeHint());
+
+        Qt::WindowFlags flags = dlg->windowFlags();
+        dlg->setWindowFlags(flags | Qt::Tool | Qt::Dialog);
+        dlg->show();
+
+    }
+    else if (filterLayer && !multipleLayersSelected){
+        QString basePath = QFileInfo(m_view->document()->url().toLocalFile()).absolutePath();
+        QString fileNameOld = filterLayer->fileName();
+        KisFileLayer::ScalingMethod scalingMethodOld = filterLayer->scalingMethod();
+        KisDlgFileLayer dlg(basePath, filterLayer->name(), m_view->mainWindow());
+        dlg.setCaption(i18n("File Layer Properties"));
+        dlg.setFileName(fileNameOld);
+        dlg.setScalingMethod(scalingMethodOld);
 
         if (dlg.exec() == QDialog::Accepted) {
+            const QString fileNameNew = dlg.fileName();
+            KisFileLayer::ScalingMethod scalingMethodNew = dlg.scaleToImageResolution();
 
-            glayer->setName(dlg.layerName());
-
-            KisFilterConfigurationSP configAfter(dlg.configuration());
-            Q_ASSERT(configAfter);
-            QString xmlAfter = configAfter->toXML();
-
-            if(xmlBefore != xmlAfter) {
-                KisChangeFilterCmd *cmd
-                        = new KisChangeFilterCmd(glayer,
-                                                 configBefore->name(),
-                                                 xmlBefore,
-                                                 configAfter->name(),
-                                                 xmlAfter,
-                                                 true);
-                // FIXME: check whether is needed
-                cmd->redo();
-                m_view->undoAdapter()->addCommand(cmd);
-                m_view->document()->setModified(true);
+            if(fileNameNew.isEmpty()){
+                QMessageBox::critical(m_view->mainWindow(), i18nc("@title:window", "Krita"), i18n("No file name specified"));
+                return;
             }
+            filterLayer->setName(dlg.layerName());
 
+            if (fileNameOld!= fileNameNew || scalingMethodOld != scalingMethodNew) {
+                KisChangeFileLayerCmd *cmd
+                        = new KisChangeFileLayerCmd(filterLayer,
+                                                    basePath,
+                                                    fileNameOld,
+                                                    scalingMethodOld,
+                                                    basePath,
+                                                    fileNameNew,
+                                                    scalingMethodNew);
+                m_view->undoAdapter()->addCommand(cmd);
+            }
         }
     } else { // If layer == normal painting layer, vector layer, or group layer
         QList<KisNodeSP> selectedNodes = m_view->nodeManager()->selectedNodes();
@@ -339,19 +347,48 @@ void KisLayerManager::layerProperties()
         dialog->resize(dialog->minimumSizeHint());
         dialog->setAttribute(Qt::WA_DeleteOnClose);
         Qt::WindowFlags flags = dialog->windowFlags();
-        dialog->setWindowFlags(flags | Qt::WindowStaysOnTopHint | Qt::Dialog);
+        dialog->setWindowFlags(flags | Qt::Tool | Qt::Dialog);
         dialog->show();
 
     }
 }
 
+void KisLayerManager::changeCloneSource()
+{
+    QList<KisNodeSP> selectedNodes = m_view->nodeManager()->selectedNodes();
+    if (selectedNodes.isEmpty()) {
+        return;
+    }
+
+    QList<KisCloneLayerSP> cloneLayers;
+    KisNodeSP node;
+    Q_FOREACH (node, selectedNodes) {
+        KisCloneLayerSP cloneLayer(qobject_cast<KisCloneLayer *>(node.data()));
+        if (cloneLayer) {
+            cloneLayers << cloneLayer;
+        }
+    }
+
+    if (cloneLayers.isEmpty()) {
+        return;
+    }
+
+    KisDlgChangeCloneSource *dialog = new KisDlgChangeCloneSource(cloneLayers, m_view);
+    dialog->setCaption(i18n("Change Clone Layer"));
+    dialog->resize(dialog->minimumSizeHint());
+    dialog->setAttribute(Qt::WA_DeleteOnClose);
+    Qt::WindowFlags flags = dialog->windowFlags();
+    dialog->setWindowFlags(flags | Qt::Tool | Qt::Dialog);
+    dialog->show();
+}
+
 void KisLayerManager::convertNodeToPaintLayer(KisNodeSP source)
 {
     KisImageWSP image = m_view->image();
     if (!image) return;
 
 
-    KisLayer *srcLayer = dynamic_cast<KisLayer*>(source.data());
+    KisLayer *srcLayer = qobject_cast<KisLayer*>(source.data());
     if (srcLayer && (srcLayer->inherits("KisGroupLayer") || srcLayer->layerStyle() || srcLayer->childCount() > 0)) {
         image->flattenLayer(srcLayer);
         return;
@@ -359,7 +396,7 @@ void KisLayerManager::convertNodeToPaintLayer(KisNodeSP source)
 
 
     KisPaintDeviceSP srcDevice =
-        source->paintDevice() ? source->projection() : source->original();
+            source->paintDevice() ? source->projection() : source->original();
 
     bool putBehind = false;
     QString newCompositeOp = source->compositeOpId();
@@ -377,7 +414,7 @@ void KisLayerManager::convertNodeToPaintLayer(KisNodeSP source)
     KisPaintDeviceSP clone;
 
     if (*srcDevice->colorSpace() !=
-        *srcDevice->compositionSourceColorSpace()) {
+            *srcDevice->compositionSourceColorSpace()) {
 
         clone = new KisPaintDevice(srcDevice->compositionSourceColorSpace());
 
@@ -395,10 +432,10 @@ void KisLayerManager::convertNodeToPaintLayer(KisNodeSP source)
     layer->setCompositeOpId(newCompositeOp);
 
     KisNodeSP parent = source->parent();
-    KisNodeSP above = source;
+    KisNodeSP above = source->prevSibling();
 
     while (parent && !parent->allowAsChild(layer)) {
-        above = above->parent();
+        above = above ? above->parent() : source->parent();
         parent = above ? above->parent() : 0;
     }
 
@@ -407,8 +444,8 @@ void KisLayerManager::convertNodeToPaintLayer(KisNodeSP source)
     }
 
     m_commandsAdapter->beginMacro(kundo2_i18n("Convert to a Paint Layer"));
-    m_commandsAdapter->addNode(layer, parent, above);
     m_commandsAdapter->removeNode(source);
+    m_commandsAdapter->addNode(layer, parent, above);
     m_commandsAdapter->endMacro();
 
 }
@@ -421,7 +458,7 @@ void KisLayerManager::convertGroupToAnimated()
     KisPaintLayerSP animatedLayer = new KisPaintLayer(m_view->image(), group->name(), OPACITY_OPAQUE_U8);
     animatedLayer->enableAnimation();
     KisRasterKeyframeChannel *contentChannel = dynamic_cast<KisRasterKeyframeChannel*>(
-            animatedLayer->getKeyframeChannel(KisKeyframeChannel::Content.id(), true));
+                animatedLayer->getKeyframeChannel(KisKeyframeChannel::Content.id(), true));
     KIS_ASSERT_RECOVER_RETURN(contentChannel);
 
     KisNodeSP child = group->firstChild();
@@ -439,6 +476,97 @@ void KisLayerManager::convertGroupToAnimated()
     m_commandsAdapter->endMacro();
 }
 
+void KisLayerManager::convertLayerToFileLayer(KisNodeSP source)
+{
+    KisImageSP image = m_view->image();
+    if (!image) return;
+
+    QStringList listMimeFilter = KisImportExportManager::supportedMimeTypes(KisImportExportManager::Export);
+
+    KoDialog dlg;
+    QWidget *page = new QWidget(&dlg);
+    dlg.setMainWidget(page);
+    QBoxLayout *layout = new QVBoxLayout(page);
+    dlg.setWindowTitle(i18n("Save layers to..."));
+    QLabel *lbl = new QLabel(i18n("Choose the location where the layer will be saved to. The new file layer will then reference this location."));
+    lbl->setWordWrap(true);
+    layout->addWidget(lbl);
+    KisFileNameRequester *urlRequester = new KisFileNameRequester(page);
+    urlRequester->setMode(KoFileDialog::SaveFile);
+    urlRequester->setMimeTypeFilters(listMimeFilter);
+    urlRequester->setFileName(m_view->document()->url().toLocalFile());
+    if (m_view->document()->url().isLocalFile()) {
+        QFileInfo location = QFileInfo(m_view->document()->url().toLocalFile()).completeBaseName();
+        location.setFile(location.dir(), location.completeBaseName() + "_" + source->name() + ".png");
+        urlRequester->setFileName(location.absoluteFilePath());
+    }
+    else {
+        const QFileInfo location = QFileInfo(QStandardPaths::writableLocation(QStandardPaths::HomeLocation));
+        const QString proposedFileName = QDir(location.absoluteFilePath()).absoluteFilePath(source->name() + ".png");
+        urlRequester->setFileName(proposedFileName);
+    }
+
+    // We don't want .kra files as file layers, Krita cannot handle the load.
+    QStringList mimes = KisImportExportManager::supportedMimeTypes(KisImportExportManager::Export);
+    int i = mimes.indexOf(KIS_MIME_TYPE);
+    if (i >= 0 && i < mimes.size()) {
+        mimes.removeAt(i);
+    }
+    urlRequester->setMimeTypeFilters(mimes);
+
+    layout->addWidget(urlRequester);
+    if (!dlg.exec()) return;
+
+    QString path = urlRequester->fileName();
+
+    if (path.isEmpty()) return;
+
+    QFileInfo f(path);
+
+    QString mimeType= KisMimeDatabase::mimeTypeForFile(f.fileName());
+    if (mimeType.isEmpty()) {
+        mimeType = "image/png";
+    }
+    QScopedPointer<KisDocument> doc(KisPart::instance()->createDocument());
+
+    QRect bounds = source->exactBounds();
+    if (bounds.isEmpty()) {
+        bounds = image->bounds();
+    }
+    KisImageSP dst = new KisImage(doc->createUndoStore(),
+                                  image->width(),
+                                  image->height(),
+                                  image->projection()->compositionSourceColorSpace(),
+                                  source->name());
+    dst->setResolution(image->xRes(), image->yRes());
+    doc->setFileBatchMode(false);
+    doc->setCurrentImage(dst);
+    KisNodeSP node = source->clone();
+    dst->addNode(node);
+    dst->initialRefreshGraph();
+    dst->cropImage(bounds);
+    dst->waitForDone();
+
+    bool r = doc->exportDocumentSync(QUrl::fromLocalFile(path), mimeType.toLatin1());
+    if (!r) {
+
+        qWarning() << "Converting layer to file layer. path:"<< path << "gave errors" << doc->errorMessage();
+    } else {
+        QString basePath = QFileInfo(m_view->document()->url().toLocalFile()).absolutePath();
+        QString relativePath = QDir(basePath).relativeFilePath(path);
+        KisFileLayer *fileLayer = new KisFileLayer(image, basePath, relativePath, KisFileLayer::None, source->name(), OPACITY_OPAQUE_U8);
+        fileLayer->setX(bounds.x());
+        fileLayer->setY(bounds.y());
+        KisNodeSP dstParent = source->parent();
+        KisNodeSP dstAboveThis = source->prevSibling();
+        m_commandsAdapter->beginMacro(kundo2_i18n("Convert to a file layer"));
+        m_commandsAdapter->removeNode(source);
+        m_commandsAdapter->addNode(fileLayer, dstParent, dstAboveThis);
+        m_commandsAdapter->endMacro();
+    }
+    doc->closeUrl(false);
+}
+
 void KisLayerManager::adjustLayerPosition(KisNodeSP node, KisNodeSP activeNode, KisNodeSP &parent, KisNodeSP &above)
 {
     Q_ASSERT(activeNode);
@@ -446,6 +574,12 @@ void KisLayerManager::adjustLayerPosition(KisNodeSP node, KisNodeSP activeNode,
     parent = activeNode;
     above = parent->lastChild();
 
+    if (parent->inherits("KisGroupLayer") && parent->collapsed()) {
+        above = parent;
+        parent = parent->parent();
+        return;
+    }
+
     while (parent &&
            (!parent->allowAsChild(node) || parent->userLocked())) {
 
@@ -455,71 +589,90 @@ void KisLayerManager::adjustLayerPosition(KisNodeSP node, KisNodeSP activeNode,
 
     if (!parent) {
         warnKrita << "KisLayerManager::adjustLayerPosition:"
-                   << "No node accepted newly created node";
+                  << "No node accepted newly created node";
 
         parent = m_view->image()->root();
         above = parent->lastChild();
     }
 }
 
-void KisLayerManager::addLayerCommon(KisNodeSP activeNode, KisLayerSP layer, bool updateImage)
+void KisLayerManager::addLayerCommon(KisNodeSP activeNode, KisNodeSP layer, bool updateImage, KisProcessingApplicator *applicator)
 {
     KisNodeSP parent;
     KisNodeSP above;
     adjustLayerPosition(layer, activeNode, parent, above);
 
-
     KisGroupLayer *group = dynamic_cast<KisGroupLayer*>(parent.data());
     const bool parentForceUpdate = group && !group->projectionIsValid();
     updateImage |= parentForceUpdate;
 
-    m_commandsAdapter->addNode(layer, parent, above, updateImage, updateImage);
+    m_commandsAdapter->addNodeAsync(layer, parent, above, updateImage, updateImage, applicator);
 }
 
-KisLayerSP KisLayerManager::addLayer(KisNodeSP activeNode)
+KisLayerSP KisLayerManager::addPaintLayer(KisNodeSP activeNode)
 {
-    KisLayerSP layer = KisLayerUtils::constructDefaultLayer(m_view->image());
-    addLayerCommon(activeNode, layer, false);
+    KisImageWSP image = m_view->image();
+    KisLayerSP layer = new KisPaintLayer(image.data(), image->nextLayerName(), OPACITY_OPAQUE_U8, image->colorSpace());
+    addLayerCommon(activeNode, layer, false, 0);
 
     return layer;
 }
 
-void KisLayerManager::addGroupLayer(KisNodeSP activeNode)
+KisNodeSP KisLayerManager::addGroupLayer(KisNodeSP activeNode)
 {
     KisImageWSP image = m_view->image();
-    addLayerCommon(activeNode,
-                   new KisGroupLayer(image.data(), image->nextLayerName(), OPACITY_OPAQUE_U8), false);
+    KisGroupLayerSP group = new KisGroupLayer(image.data(), image->nextLayerName(), OPACITY_OPAQUE_U8);
+    addLayerCommon(activeNode, group, false, 0);
+    return group;
 }
 
-void KisLayerManager::addCloneLayer(KisNodeSP activeNode)
+KisNodeSP KisLayerManager::addCloneLayer(KisNodeList nodes)
 {
     KisImageWSP image = m_view->image();
-    addLayerCommon(activeNode,
-                   new KisCloneLayer(activeLayer(), image.data(), image->nextLayerName(), OPACITY_OPAQUE_U8));
+
+    KisNodeList filteredNodes = KisLayerUtils::sortAndFilterMergableInternalNodes(nodes, false);
+    if (filteredNodes.isEmpty()) return KisNodeSP();
+
+    KisNodeSP newAbove = filteredNodes.last();
+
+    KisNodeSP node, lastClonedNode;
+    Q_FOREACH (node, filteredNodes) {
+        lastClonedNode = new KisCloneLayer(qobject_cast<KisLayer*>(node.data()), image.data(), image->nextLayerName(), OPACITY_OPAQUE_U8);
+        addLayerCommon(newAbove, lastClonedNode, true, 0 );
+    }
+
+    return lastClonedNode;
 }
 
-void KisLayerManager::addShapeLayer(KisNodeSP activeNode)
+KisNodeSP KisLayerManager::addShapeLayer(KisNodeSP activeNode)
 {
-    if (!m_view) return;
-    if (!m_view->document()) return;
+    if (!m_view) return 0;
+    if (!m_view->document()) return 0;
 
     KisImageWSP image = m_view->image();
     KisShapeLayerSP layer = new KisShapeLayer(m_view->document()->shapeController(), image.data(), image->nextLayerName(), OPACITY_OPAQUE_U8);
 
-    addLayerCommon(activeNode, layer, false);
+    addLayerCommon(activeNode, layer, false, 0);
+
+    return layer;
 }
 
-void KisLayerManager::addAdjustmentLayer(KisNodeSP activeNode)
+KisNodeSP KisLayerManager::addAdjustmentLayer(KisNodeSP activeNode)
 {
     KisImageWSP image = m_view->image();
 
     KisSelectionSP selection = m_view->selection();
-    KisAdjustmentLayerSP adjl = addAdjustmentLayer(activeNode, QString(), 0, selection);
-    image->refreshGraph();
+
+    KisProcessingApplicator applicator(image, 0, KisProcessingApplicator::NONE,
+                                       KisImageSignalVector() << ModifiedSignal,
+                                       kundo2_i18n("Add Layer"));
+
+
+    KisAdjustmentLayerSP adjl = addAdjustmentLayer(activeNode, QString(), 0, selection, &applicator);
 
     KisPaintDeviceSP previewDevice = new KisPaintDevice(*adjl->original());
 
-    KisDlgAdjustmentLayer dlg(adjl, adjl.data(), previewDevice, image->nextLayerName(), i18n("New Filter Layer"), m_view);
+    KisDlgAdjustmentLayer dlg(adjl, adjl.data(), previewDevice, image->nextLayerName(), i18n("New Filter Layer"), m_view, qApp->activeWindow());
     dlg.resize(dlg.minimumSizeHint());
 
     // ensure that the device may be free'd by the dialog
@@ -528,70 +681,58 @@ void KisLayerManager::addAdjustmentLayer(KisNodeSP activeNode)
 
     if (dlg.exec() != QDialog::Accepted || adjl->filter().isNull()) {
         // XXX: add messagebox warning if there's no filter set!
-        m_commandsAdapter->undoLastCommand();
+        applicator.cancel();
     } else {
         adjl->setName(dlg.layerName());
+        applicator.end();
     }
 
+    return adjl;
 }
 
 KisAdjustmentLayerSP KisLayerManager::addAdjustmentLayer(KisNodeSP activeNode, const QString & name,
-                                                         KisFilterConfigurationSP  filter, KisSelectionSP selection)
+                                                         KisFilterConfigurationSP  filter,
+                                                         KisSelectionSP selection,
+                                                         KisProcessingApplicator *applicator)
 {
     KisImageWSP image = m_view->image();
     KisAdjustmentLayerSP layer = new KisAdjustmentLayer(image, name, filter, selection);
-    addLayerCommon(activeNode, layer);
+    addLayerCommon(activeNode, layer, true, applicator);
 
     return layer;
 }
 
-void KisLayerManager::addGeneratorLayer(KisNodeSP activeNode)
+KisNodeSP KisLayerManager::addGeneratorLayer(KisNodeSP activeNode)
 {
     KisImageWSP image = m_view->image();
+    QColor currentForeground = m_view->canvasResourceProvider()->fgColor().toQColor();
+
+
+    KisDlgGeneratorLayer dlg(image->nextLayerName(), m_view, m_view->mainWindow(), 0, 0);
+    KisFilterConfigurationSP defaultConfig = dlg.configuration();
+    defaultConfig->setProperty("color", currentForeground);
+    dlg.setConfiguration(defaultConfig);
 
-    KisDlgGeneratorLayer dlg(image->nextLayerName(), m_view, m_view->mainWindow());
     dlg.resize(dlg.minimumSizeHint());
-
     if (dlg.exec() == QDialog::Accepted) {
         KisSelectionSP selection = m_view->selection();
         KisFilterConfigurationSP  generator = dlg.configuration();
         QString name = dlg.layerName();
 
-        addLayerCommon(activeNode,
-            new KisGeneratorLayer(image, name, generator, selection));
+        KisNodeSP node = new KisGeneratorLayer(image, name, generator, selection);
+
+        addLayerCommon(activeNode, node, true, 0);
+
+        return node;
     }
-
-}
-
-void KisLayerManager::rotateLayer(double radians)
-{
-    if (!m_view->image()) return;
-
-    KisLayerSP layer = activeLayer();
-    if (!layer) return;
-
-    if (!m_view->blockUntillOperationsFinished(m_view->image())) return;
-
-    m_view->image()->rotateNode(layer, radians);
-}
-
-void KisLayerManager::shearLayer(double angleX, double angleY)
-{
-    if (!m_view->image()) return;
-
-    KisLayerSP layer = activeLayer();
-    if (!layer) return;
-
-    if (!m_view->blockUntillOperationsFinished(m_view->image())) return;
-
-    m_view->image()->shearNode(layer, angleX, angleY);
+    return 0;
 }
 
 void KisLayerManager::flattenImage()
 {
     KisImageSP image = m_view->image();
 
-    if (!m_view->blockUntillOperationsFinished(image)) return;
+    if (!m_view->blockUntilOperationsFinished(image)) return;
 
     if (image) {
         bool doIt = true;
@@ -609,7 +750,7 @@ void KisLayerManager::flattenImage()
         }
 
         if (doIt) {
-            image->flatten();
+            image->flatten(m_view->activeNode());
         }
     }
 }
@@ -624,7 +765,7 @@ bool tryMergeSelectionMasks(KisNodeSP currentNode, KisImageSP image)
 
     KisNodeSP prevNode = currentNode->prevSibling();
     if (isSelectionMask(currentNode) &&
-        prevNode && isSelectionMask(prevNode)) {
+            prevNode && isSelectionMask(prevNode)) {
 
         QList<KisNodeSP> mergedNodes;
         mergedNodes.append(currentNode);
@@ -661,27 +802,27 @@ void KisLayerManager::mergeLayer()
     KisLayerSP layer = activeLayer();
     if (!layer) return;
 
-    if (!m_view->blockUntillOperationsFinished(image)) return;
+    if (!m_view->blockUntilOperationsFinished(image)) return;
 
     QList<KisNodeSP> selectedNodes = m_view->nodeManager()->selectedNodes();
-      if (selectedNodes.size() > 1) {
+    if (selectedNodes.size() > 1) {
         image->mergeMultipleLayers(selectedNodes, m_view->activeNode());
     }
 
-      else if (tryMergeSelectionMasks(m_view->activeNode(), image)) {
+    else if (tryMergeSelectionMasks(m_view->activeNode(), image)) {
         // already done!
     } else if (tryFlattenGroupLayer(m_view->activeNode(), image)) {
         // already done!
     } else {
 
         if (!layer->prevSibling()) return;
-        KisLayer *prevLayer = dynamic_cast<KisLayer*>(layer->prevSibling().data());
+        KisLayer *prevLayer = qobject_cast<KisLayer*>(layer->prevSibling().data());
         if (!prevLayer) return;
         if (prevLayer->userLocked()) {
             m_view->showFloatingMessage(
-                i18nc("floating message in layer manager",
-                      "Layer is locked "),
-                QIcon(), 2000, KisFloatingMessage::Low);
+                        i18nc("floating message in layer manager",
+                              "Layer is locked "),
+                        QIcon(), 2000, KisFloatingMessage::Low);
         }
 
         else if (layer->metaData()->isEmpty() && prevLayer->metaData()->isEmpty()) {
@@ -705,7 +846,7 @@ void KisLayerManager::flattenLayer()
     KisLayerSP layer = activeLayer();
     if (!layer) return;
 
-    if (!m_view->blockUntillOperationsFinished(image)) return;
+    if (!m_view->blockUntilOperationsFinished(image)) return;
 
     convertNodeToPaintLayer(layer);
     m_view->updateGUI();
@@ -719,7 +860,7 @@ void KisLayerManager::rasterizeLayer()
     KisLayerSP layer = activeLayer();
     if (!layer) return;
 
-    if (!m_view->blockUntillOperationsFinished(image)) return;
+    if (!m_view->blockUntilOperationsFinished(image)) return;
 
     KisPaintLayerSP paintLayer = new KisPaintLayer(image, layer->name(), layer->opacity());
     KisPainter gc(paintLayer->paintDevice());
@@ -749,7 +890,7 @@ void KisLayerManager::layersUpdated()
 
 void KisLayerManager::saveGroupLayers()
 {
-    QStringList listMimeFilter = KisImportExportManager::mimeFilter(KisImportExportManager::Export);
+    QStringList listMimeFilter = KisImportExportManager::supportedMimeTypes(KisImportExportManager::Export);
 
     KoDialog dlg;
     QWidget *page = new QWidget(&dlg);
@@ -780,12 +921,12 @@ void KisLayerManager::saveGroupLayers()
 
     QFileInfo f(path);
 
-    QString mimeType= KisMimeDatabase::mimeTypeForFile(f.fileName());
+    QString mimeType= KisMimeDatabase::mimeTypeForFile(f.fileName(), false);
     if (mimeType.isEmpty()) {
         mimeType = "image/png";
     }
     QString extension = KisMimeDatabase::suffixesForMimeType(mimeType).first();
-    QString basename = f.baseName();
+    QString basename = f.completeBaseName();
 
     KisImageSP image = m_view->image();
     if (!image) return;
@@ -800,9 +941,8 @@ bool KisLayerManager::activeLayerHasSelection()
     return (activeLayer()->selection() != 0);
 }
 
-void KisLayerManager::addFileLayer(KisNodeSP activeNode)
+KisNodeSP KisLayerManager::addFileLayer(KisNodeSP activeNode)
 {
-
     QString basePath;
     QUrl url = m_view->document()->url();
     if (url.isLocalFile()) {
@@ -819,15 +959,15 @@ void KisLayerManager::addFileLayer(KisNodeSP activeNode)
 
         if(fileName.isEmpty()){
             QMessageBox::critical(m_view->mainWindow(), i18nc("@title:window", "Krita"), i18n("No file name specified"));
-            return;
+            return 0;
         }
 
         KisFileLayer::ScalingMethod scalingMethod = dlg.scaleToImageResolution();
-
-        addLayerCommon(activeNode,
-                       new KisFileLayer(image, basePath, fileName, scalingMethod, name, OPACITY_OPAQUE_U8));
+        KisNodeSP node = new KisFileLayer(image, basePath, fileName, scalingMethod, name, OPACITY_OPAQUE_U8);
+        addLayerCommon(activeNode, node, true, 0);
+        return node;
     }
-
+    return 0;
 }
 
 void updateLayerStyles(KisLayerSP layer, KisDlgLayerStyle *dlg)
@@ -843,7 +983,7 @@ void KisLayerManager::layerStyle()
     KisLayerSP layer = activeLayer();
     if (!layer) return;
 
-    if (!m_view->blockUntillOperationsFinished(image)) return;
+    if (!m_view->blockUntilOperationsFinished(image)) return;
 
     KisPSDLayerStyleSP oldStyle;
     if (layer->layerStyle()) {
@@ -853,7 +993,7 @@ void KisLayerManager::layerStyle()
         oldStyle = toQShared(new KisPSDLayerStyle());
     }
 
-    KisDlgLayerStyle dlg(oldStyle->clone(), m_view->resourceProvider());
+    KisDlgLayerStyle dlg(oldStyle->clone(), m_view->canvasResourceProvider());
 
     std::function<void ()> updateCall(std::bind(updateLayerStyles, layer, &dlg));
     SignalToFunctionProxy proxy(updateCall);
@@ -863,7 +1003,7 @@ void KisLayerManager::layerStyle()
         KisPSDLayerStyleSP newStyle = dlg.style();
 
         KUndo2CommandSP command = toQShared(
-            new KisSetLayerStyleCommand(layer, oldStyle, newStyle));
+                    new KisSetLayerStyleCommand(layer, oldStyle, newStyle));
 
         image->postExecutionUndoAdapter()->addCommand(command);
     }
