@@ -201,14 +201,13 @@ public:
     KisSignalCompressor shapeChangedCompressor;
     KoFlake::FillVariant fillVariant;
 
-
-    QList<KoShape*> previousShapeSelected;/// container to see if the selection has actually changed
-
     bool noSelectionTrackingMode;
 
     Ui_KoFillConfigWidget *ui;
 
     std::vector<KisAcyclicSignalConnector::Blocker> deactivationLocks;
+
+    boost::optional<KoColor> overriddenColorFromProvider;
 };
 
 KoFillConfigWidget::KoFillConfigWidget(KoCanvasBase *canvas, KoFlake::FillVariant fillVariant, bool trackShapeSelection, QWidget *parent)
@@ -239,6 +238,10 @@ KoFillConfigWidget::KoFillConfigWidget(KoCanvasBase *canvas, KoFlake::FillVarian
          this, SIGNAL(sigInternalRequestColorToResourceManager()),
          this, SLOT(slotProposeCurrentColorToResourceManager()));
 
+    KisAcyclicSignalConnector *resetConnector = d->resourceManagerAcyclicConnector.createCoordinatedConnector();
+    resetConnector->connectForwardVoid(
+         this, SIGNAL(sigInternalRecoverColorInResourceManager()),
+         this, SLOT(slotRecoverColorInResourceManager()));
 
     // configure GUI
 
@@ -348,6 +351,8 @@ void KoFillConfigWidget::activate()
 
 void KoFillConfigWidget::deactivate()
 {
+    emit sigInternalRecoverColorInResourceManager();
+
     KIS_SAFE_ASSERT_RECOVER_RETURN(d->deactivationLocks.empty());
     d->deactivationLocks.push_back(KisAcyclicSignalConnector::Blocker(d->shapeChangedAcyclicConnector));
     d->deactivationLocks.push_back(KisAcyclicSignalConnector::Blocker(d->resourceManagerAcyclicConnector));
@@ -518,6 +523,8 @@ void KoFillConfigWidget::colorChanged()
         return;
     }
 
+    d->overriddenColorFromProvider = boost::none;
+
     KoShapeFillWrapper wrapper(selectedShapes, d->fillVariant);
 
 
@@ -573,6 +580,11 @@ void KoFillConfigWidget::slotProposeCurrentColorToResourceManager()
     }
 
     if (hasColor) {
+        if (!d->overriddenColorFromProvider) {
+            d->overriddenColorFromProvider =
+                d->canvas->resourceManager()->resource(colorSlot).value<KoColor>();
+        }
+
         /**
          * Don't let opacity leak to our resource manager system
          *
@@ -584,6 +596,19 @@ void KoFillConfigWidget::slotProposeCurrentColorToResourceManager()
     }
 }
 
+void KoFillConfigWidget::slotRecoverColorInResourceManager()
+{
+    if (d->overriddenColorFromProvider) {
+        KoCanvasResourceProvider::CanvasResource colorSlot = KoCanvasResourceProvider::ForegroundColor;
+        if (d->fillVariant == KoFlake::StrokeFill) {
+            colorSlot = KoCanvasResourceProvider::BackgroundColor;
+        }
+
+        d->canvas->resourceManager()->setResource(colorSlot, QVariant::fromValue(*d->overriddenColorFromProvider));
+        d->overriddenColorFromProvider = boost::none;
+    }
+}
+
 template <class ResourceServer>
 QString findFirstAvailableResourceName(const QString &baseName, ResourceServer *server)
 {
@@ -770,12 +795,7 @@ void KoFillConfigWidget::shapeChanged()
 
     QList<KoShape*> shapes = currentShapes();
 
-    // check to see if the shape actually changed...or is still the same shape
-    if (d->previousShapeSelected == shapes) {
-        return;
-    } else {
-        d->previousShapeSelected = shapes;
-    }
+    bool shouldUploadColorToResourceManager = false;
 
     if (shapes.isEmpty() ||
         (shapes.size() > 1 && KoShapeFillWrapper(shapes, d->fillVariant).isMixedFill())) {
@@ -793,6 +813,8 @@ void KoFillConfigWidget::shapeChanged()
         KoShape *shape = shapes.first();
         updateFillIndexFromShape(shape);
         updateFillColorFromShape(shape); // updates tool options fields
+
+        shouldUploadColorToResourceManager = true;
     }
 
     // updates the UI
@@ -800,6 +822,12 @@ void KoFillConfigWidget::shapeChanged()
 
     updateWidgetComponentVisbility();
     slotUpdateFillTitle();
+
+    if (shouldUploadColorToResourceManager) {
+        emit sigInternalRequestColorToResourceManager();
+    } else {
+        emit sigInternalRecoverColorInResourceManager();
+    }
 }
 
 void KoFillConfigWidget::updateFillIndexFromShape(KoShape *shape)
