@@ -93,8 +93,8 @@ KoResourceTaggingManager::KoResourceTaggingManager(KoResourceModel *model, QWidg
             this, SLOT(contextCreateNewTag(QString)));
     connect(d->tagChooser, SIGNAL(tagDeletionRequested(QString)),
             this, SLOT(removeTagFromComboBox(QString)));
-    connect(d->tagChooser, SIGNAL(tagRenamingRequested(QString, QString)),
-            this, SLOT(renameTag(QString, QString)));
+    connect(d->tagChooser, SIGNAL(tagRenamingRequested(QString,QString)),
+            this, SLOT(renameTag(QString,QString)));
     connect(d->tagChooser, SIGNAL(tagUndeletionRequested(QString)),
             this, SLOT(undeleteTag(QString)));
     connect(d->tagChooser, SIGNAL(tagUndeletionListPurgeRequested()),
@@ -362,14 +362,14 @@ void KoResourceTaggingManager::contextMenuRequested(KoResource* resource, const
                                           d->tagChooser->currentlySelectedTag(),
                                           d->tagChooser->allTags());
 
-    connect(&menu, SIGNAL(resourceTagAdditionRequested(KoResource*, QString)),
-            this, SLOT(contextAddTagToResource(KoResource*, QString)));
+    connect(&menu, SIGNAL(resourceTagAdditionRequested(KoResource*,QString)),
+            this, SLOT(contextAddTagToResource(KoResource*,QString)));
 
-    connect(&menu, SIGNAL(resourceTagRemovalRequested(KoResource*, QString)),
-            this, SLOT(contextRemoveTagFromResource(KoResource*, QString)));
+    connect(&menu, SIGNAL(resourceTagRemovalRequested(KoResource*,QString)),
+            this, SLOT(contextRemoveTagFromResource(KoResource*,QString)));
 
-    connect(&menu, SIGNAL(resourceAssignmentToNewTagRequested(KoResource*, QString)),
-            this, SLOT(contextCreateNewTag(KoResource*, QString)));
+    connect(&menu, SIGNAL(resourceAssignmentToNewTagRequested(KoResource*,QString)),
+            this, SLOT(contextCreateNewTag(KoResource*,QString)));
     menu.exec(pos);
 }
 
@@ -385,11 +385,6 @@ KoTagChooserWidget* KoResourceTaggingManager::tagChooserWidget()
     return d->tagChooser;
 }
 
-QLineEdit* KoResourceTaggingManager::searchField()
-{
-    return d->tagFilter->searchField();
-}
-
 KoTagFilterWidget* KoResourceTaggingManager::tagFilterWidget()
 {
     return d->tagFilter;
