@@ -35,6 +35,8 @@
 #include <QMenu>
 #include <QMouseEvent>
 #include <QFontDatabase>
+#include <QtMath>
+#include <cmath>
 
 #include <KoViewConverter.h>
 
@@ -79,11 +81,11 @@ void RulerTabChooser::paintEvent(QPaintEvent *)
     QPainter painter(this);
     QPolygonF polygon;
 
-    painter.setPen(palette().color(QPalette::Text));
+    painter.setPen(QPen(palette().color(QPalette::Text), 0));
     painter.setBrush(palette().color(QPalette::Text));
     painter.setRenderHint( QPainter::Antialiasing );
 
-    qreal x= width()/2;
+    qreal x = 0.5 * width();
     painter.translate(0,-height()/2+5);
 
     switch (m_type) {
@@ -118,10 +120,13 @@ void RulerTabChooser::paintEvent(QPaintEvent *)
     }
 }
 
-static int compareTabs(KoRuler::Tab &tab1, KoRuler::Tab &tab2)
-{
-    return tab1.position < tab2.position;
-}
+struct {
+    bool operator()(KoRuler::Tab tab1, KoRuler::Tab tab2) const
+    {
+        return tab1.position < tab2.position;
+    }
+} compareTabs;
+
 
 QRectF HorizontalPaintingStrategy::drawBackground(const KoRulerPrivate *d, QPainter &painter)
 {
@@ -140,7 +145,10 @@ QRectF HorizontalPaintingStrategy::drawBackground(const KoRulerPrivate *d, QPain
           d->viewConverter->documentToViewX(d->effectiveActiveRangeEnd()) + d->offset));
     activeRangeRectangle.setHeight(rectangle.height() - 2);
 
-    painter.setPen(d->ruler->palette().color(QPalette::Mid));
+    painter.setPen(QPen(d->ruler->palette().color(QPalette::Mid), 0));
+
+
+    painter.fillRect(rectangle,d->ruler->palette().color(QPalette::AlternateBase)); // make background slightly different so it is easier to see
     painter.drawRect(rectangle);
 
     if(d->effectiveActiveRangeStart() != d->effectiveActiveRangeEnd())
@@ -169,7 +177,7 @@ void HorizontalPaintingStrategy::drawTabs(const KoRulerPrivate *d, QPainter &pai
     QPolygonF polygon;
 
     const QColor tabColor = d->ruler->palette().color(QPalette::Text);
-    painter.setPen(tabColor);
+    painter.setPen(QPen(tabColor, 0));
     painter.setBrush(tabColor);
     painter.setRenderHint( QPainter::Antialiasing );
 
@@ -252,126 +260,127 @@ void HorizontalPaintingStrategy::drawTabs(const KoRulerPrivate *d, QPainter &pai
 
 void HorizontalPaintingStrategy::drawMeasurements(const KoRulerPrivate *d, QPainter &painter, const QRectF &rectangle)
 {
-    qreal numberStep = d->numberStepForUnit(); // number step in unit
-//    QRectF activeRangeRectangle;
-    int numberStepPixel = qRound(d->viewConverter->documentToViewX(d->unit.fromUserValue(numberStep)));
-//    const bool adjustMillimeters = (d->unit.type() == KoUnit::Millimeter);
-
     const QFont font = QFontDatabase::systemFont(QFontDatabase::SmallestReadableFont);
     const QFontMetrics fontMetrics(font);
+    const QPen numberPen(d->ruler->palette().color(QPalette::Text), 0);
+    const QPen markerPen(d->ruler->palette().color(QPalette::Inactive, QPalette::Text), 0);
+    painter.setPen(markerPen);
     painter.setFont(font);
 
-    if (numberStepPixel == 0 || numberStep == 0)
-        return;
-
-    // Calc the longest text length
-    int textLength = 0;
-    for(int i = 0; i < lengthInPixel; i += numberStepPixel) {
-        int number = qRound((i / numberStepPixel) * numberStep);
-
-        textLength = qMax(textLength, fontMetrics.width(QString::number(number)));
+    // length of the ruler in pixels
+    const int rulerLengthPixel = d->viewConverter->documentToViewX(d->rulerLength);
+    // length of the ruler in the chosen unit
+    const qreal rulerLengthUnit = d->unit.toUserValue(d->rulerLength);
+    // length of a unit in pixels
+    const qreal unitLength = d->viewConverter->documentToViewX(d->unit.fromUserValue(1.0));
+    // length of the text for longest number with some padding
+    const int textLength = fontMetrics.width(QString::number(qCeil(rulerLengthUnit))) + 20;
+    // the minimal separation of numbers that won't make the text a mess
+    const qreal minimalNumberSeparation = qCeil(textLength / unitLength);
+    // the chosen separation of numbers so that we have "whole" numbers
+    qreal numberSeparation = minimalNumberSeparation;
+    if (minimalNumberSeparation <= 5) {
+    } else if (minimalNumberSeparation <= 10) {
+        numberSeparation = 10;
+    } else if (minimalNumberSeparation <= 20) {
+        numberSeparation = 20;
+    } else if (minimalNumberSeparation <= 50) {
+        numberSeparation = 50;
+    } else if (minimalNumberSeparation <= 100) {
+        numberSeparation = 100;
+    } else if (minimalNumberSeparation <= 200) {
+        numberSeparation = 200;
+    } else if (minimalNumberSeparation <= 500) {
+        numberSeparation = 500;
+    } else {
+        numberSeparation = qRound(qreal(numberSeparation / 100)) * 100;
     }
-    textLength += 4;  // Add some padding
+    // the chosen separation converted to pixel
+    const int pixelSeparation = qRound(d->viewConverter->documentToViewX(d->unit.fromUserValue(numberSeparation)));
+    // the value used to calculate displacement of secondary marks
+    const int secondarydisplacement = d->rightToLeft ? -pixelSeparation : pixelSeparation;
 
-    // Change number step so all digits fits
-    while(textLength > numberStepPixel) {
-        numberStepPixel += numberStepPixel;
-        numberStep += numberStep;
+    // Draw the marks
+    int rulerEnd;
+    qreal numberStart, numberEnd;
+    if (d->rightToLeft) {
+        rulerEnd = d->offset;
+        numberStart = d->unit.toUserValue(d->viewConverter->viewToDocumentX(
+                    rulerLengthPixel + d->offset - rectangle.right()));
+        // draw the partly hidden mark when the left part of the ruler is
+        // invisible
+        int pixelEnd = rulerLengthPixel + d->offset - rectangle.left();
+        if (rulerEnd + textLength < rectangle.left()) {
+            pixelEnd -= textLength;
+        }
+        numberEnd = d->unit.toUserValue(d->viewConverter->viewToDocumentX(
+                    pixelEnd));
+        if (rulerLengthPixel + d->offset > rectangle.right()) {
+            numberStart -= fmod(numberStart, numberSeparation);
+        }
+    } else {
+        rulerEnd = d->offset + rulerLengthPixel;
+        numberStart = d->unit.toUserValue(d->viewConverter->viewToDocumentX(
+                    rectangle.left() - d->offset));
+        // draw the partly hidden mark when the right part of the ruler is
+        // invisible
+        int pixelEnd = rectangle.right() - d->offset;
+        if (rulerEnd - textLength > rectangle.right()) {
+            pixelEnd += textLength;
+        }
+        numberEnd = d->unit.toUserValue(d->viewConverter->viewToDocumentX(
+                    pixelEnd));
+        if (d->offset < 0) {
+            numberStart -= fmod(numberStart, numberSeparation);
+        }
     }
-
-    int start=0;
-    // Calc the first number step
-    if(d->offset < 0)
-        start = qAbs(d->offset);
-
-    // make a little hack so rulers shows correctly inversed number aligned
-    const qreal lengthInUnit = d->unit.toUserValue(d->rulerLength);
-    const qreal hackyLength = lengthInUnit - fmod(lengthInUnit, numberStep);
-    if(d->rightToLeft) {
-        start -= int(d->viewConverter->documentToViewX(fmod(d->rulerLength,
-                    d->unit.fromUserValue(numberStep))));
-    }
-
-    int stepCount = (start / numberStepPixel) + 1;
-    int halfStepCount = (start / qRound(numberStepPixel * 0.5)) + 1;
-    int quarterStepCount = (start / qRound(numberStepPixel * 0.25)) + 1;
-
-    int pos = 0;
-    const QPen numberPen(d->ruler->palette().color(QPalette::Text));
-    const QPen markerPen(d->ruler->palette().color(QPalette::Inactive, QPalette::Text));
-    painter.setPen(markerPen);
-
-    if(d->offset > 0)
-        painter.translate(d->offset, 0);
-
-    const int len = qRound(rectangle.width()) + start;
-    int nextStep = qRound(d->viewConverter->documentToViewX(
-        d->unit.fromUserValue(numberStep * stepCount)));
-    int nextHalfStep = qRound(d->viewConverter->documentToViewX(d->unit.fromUserValue(
-        numberStep * 0.5 * halfStepCount)));
-    int nextQuarterStep = qRound(d->viewConverter->documentToViewX(d->unit.fromUserValue(
-        numberStep * 0.25 * quarterStepCount)));
-
-    for(int i = start; i < len; ++i) {
-        pos = i - start;
-
-        if(i == nextStep) {
-            if(pos != 0)
-                painter.drawLine(QPointF(pos, rectangle.bottom()-1),
-                                 QPointF(pos, rectangle.bottom() - fullStepMarkerLength));
-
-            int number = qRound(stepCount * numberStep);
-
-            QString numberText = QString::number(number);
-            int x = pos;
-            if (d->rightToLeft) { // this is done in a hacky way with the fine tuning done above
-                numberText = QString::number(hackyLength - stepCount * numberStep);
-            }
-            painter.setPen(numberPen);
-            painter.drawText(QPointF(x-fontMetrics.width(numberText)/2.0,
-                                     rectangle.bottom() -fullStepMarkerLength -measurementTextAboveBelowMargin),
-                             numberText);
-            painter.setPen(markerPen);
-
-            ++stepCount;
-            nextStep = qRound(d->viewConverter->documentToViewX(
-                d->unit.fromUserValue(numberStep * stepCount)));
-            ++halfStepCount;
-            nextHalfStep = qRound(d->viewConverter->documentToViewX(d->unit.fromUserValue(
-                numberStep * 0.5 * halfStepCount)));
-            ++quarterStepCount;
-            nextQuarterStep = qRound(d->viewConverter->documentToViewX(d->unit.fromUserValue(
-                numberStep * 0.25 * quarterStepCount)));
+    for(qreal n = numberStart; n < numberEnd; n += numberSeparation) {
+        int posOnRuler = qRound(d->viewConverter->documentToViewX(d->unit.fromUserValue(n)));
+        int x = posOnRuler + d->offset;
+        if (d->rightToLeft) {
+            x = d->offset + rulerLengthPixel - posOnRuler;
         }
-        else if(i == nextHalfStep) {
-            if(pos != 0)
-                painter.drawLine(QPointF(pos, rectangle.bottom()-1),
-                                 QPointF(pos, rectangle.bottom() - halfStepMarkerLength));
 
-            ++halfStepCount;
-            nextHalfStep = qRound(d->viewConverter->documentToViewX(d->unit.fromUserValue(
-                numberStep * 0.5 * halfStepCount)));
-            ++quarterStepCount;
-            nextQuarterStep = qRound(d->viewConverter->documentToViewX(d->unit.fromUserValue(
-                numberStep * 0.25 * quarterStepCount)));
-        }
-        else if(i == nextQuarterStep) {
-            if(pos != 0)
-                painter.drawLine(QPointF(pos, rectangle.bottom()-1),
-                                 QPointF(pos, rectangle.bottom() - quarterStepMarkerLength));
+        // to draw the primary mark
+        painter.drawLine(QPointF(x, rectangle.bottom()-1),
+                QPointF(x, rectangle.bottom() - fullStepMarkerLength));
+        QString numberText = QString::number(n);
+        painter.setPen(numberPen);
+        painter.drawText(QPointF(x - fontMetrics.width(numberText)/2.0,
+                    rectangle.bottom() - fullStepMarkerLength - measurementTextAboveBelowMargin),
+                numberText);
 
-            ++quarterStepCount;
-            nextQuarterStep = qRound(d->viewConverter->documentToViewX(d->unit.fromUserValue(
-                numberStep * 0.25 * quarterStepCount)));
-        }
+        // start to draw secondary marks following the primary mark
+        painter.setPen(markerPen);
+
+        int xQuarterMark1 = qRound(qreal(x) + qreal(secondarydisplacement) / 4.0);
+        if (d->rightToLeft ? xQuarterMark1 < rulerEnd : xQuarterMark1 > rulerEnd)
+            break;
+        painter.drawLine(QPointF(xQuarterMark1, rectangle.bottom() - 1),
+                QPointF(xQuarterMark1, rectangle.bottom() - quarterStepMarkerLength));
+
+        int xHalfMark = qRound(qreal(x) + qreal(secondarydisplacement) / 2.0);
+        if (d->rightToLeft ? xHalfMark < rulerEnd : xHalfMark > rulerEnd)
+            break;
+        painter.drawLine(QPointF(xHalfMark, rectangle.bottom() - 1),
+                QPointF(xHalfMark, rectangle.bottom() - halfStepMarkerLength));
+
+        int xQuarterMark2 = qRound(qreal(x) + 3.0 * qreal(secondarydisplacement) / 4.0);
+        if (d->rightToLeft ? xQuarterMark2 < rulerEnd : xQuarterMark2 > rulerEnd)
+            break;
+        painter.drawLine(QPointF(xQuarterMark2, rectangle.bottom() - 1),
+                QPointF(xQuarterMark2, rectangle.bottom() - quarterStepMarkerLength));
     }
 
     // Draw the mouse indicator
-    const int mouseCoord = d->mouseCoordinate - start;
+    const int mouseCoord = d->mouseCoordinate + d->offset;
     if (d->selected == KoRulerPrivate::None || d->selected == KoRulerPrivate::HotSpot) {
         const qreal top = rectangle.y() + 1;
         const qreal bottom = rectangle.bottom() -1;
-        if (d->selected == KoRulerPrivate::None && d->showMousePosition && mouseCoord > 0 && mouseCoord < rectangle.width() )
+        if (d->selected == KoRulerPrivate::None
+                && d->showMousePosition
+                && d->mouseCoordinate > 0
+                && d->mouseCoordinate < rulerLengthPixel)
             painter.drawLine(QPointF(mouseCoord, top), QPointF(mouseCoord, bottom));
         foreach (const KoRulerPrivate::HotSpotData & hp, d->hotspots) {
             const qreal x = d->viewConverter->documentToViewX(hp.position) + d->offset;
@@ -456,7 +465,9 @@ QRectF VerticalPaintingStrategy::drawBackground(const KoRulerPrivate *d, QPainte
     activeRangeRectangle.setBottom(qMin(rectangle.bottom() - 1,
         d->viewConverter->documentToViewY(d->effectiveActiveRangeEnd()) + d->offset));
 
-    painter.setPen(d->ruler->palette().color(QPalette::Mid));
+    painter.setPen(QPen(d->ruler->palette().color(QPalette::Mid), 0));
+
+    painter.fillRect(rectangle,d->ruler->palette().color(QPalette::AlternateBase));
     painter.drawRect(rectangle);
 
     if(d->effectiveActiveRangeStart() != d->effectiveActiveRangeEnd())
@@ -480,109 +491,106 @@ QRectF VerticalPaintingStrategy::drawBackground(const KoRulerPrivate *d, QPainte
 
 void VerticalPaintingStrategy::drawMeasurements(const KoRulerPrivate *d, QPainter &painter, const QRectF &rectangle)
 {
-    qreal numberStep = d->numberStepForUnit(); // number step in unit
-    int numberStepPixel = qRound(d->viewConverter->documentToViewY( d->unit.fromUserValue(numberStep)));
-    if (numberStepPixel <= 0)
-        return;
-
     const QFont font = QFontDatabase::systemFont(QFontDatabase::SmallestReadableFont);
     const QFontMetrics fontMetrics(font);
+    const QPen numberPen(d->ruler->palette().color(QPalette::Text), 0);
+    const QPen markerPen(d->ruler->palette().color(QPalette::Inactive, QPalette::Text), 0);
+    painter.setPen(markerPen);
     painter.setFont(font);
 
-    // Calc the longest text length
-    int textLength = 0;
-
-    for(int i = 0; i < lengthInPixel; i += numberStepPixel) {
-        int number = qRound((i / numberStepPixel) * numberStep);
-        textLength = qMax(textLength, fontMetrics.width(QString::number(number)));
+    // length of the ruler in pixels
+    const int rulerLengthPixel = d->viewConverter->documentToViewY(d->rulerLength);
+    // length of the ruler in the chosen unit
+    const qreal rulerLengthUnit = d->unit.toUserValue(d->rulerLength);
+    // length of a unit in pixels
+    const qreal unitLength = d->viewConverter->documentToViewY(d->unit.fromUserValue(1.0));
+    // length of the text for longest number with some padding
+    const int textLength = fontMetrics.width(QString::number(qCeil(rulerLengthUnit))) + 20;
+    // the minimal separation of numbers that won't make the text a mess
+    const qreal minimalNumberSeparation = qCeil(textLength / unitLength);
+    // the chosen separation of numbers so that we have "whole" numbers
+    qreal numberSeparation = minimalNumberSeparation;
+    if (minimalNumberSeparation <= 5) {
+    } else if (minimalNumberSeparation <= 10) {
+        numberSeparation = 10;
+    } else if (minimalNumberSeparation <= 20) {
+        numberSeparation = 20;
+    } else if (minimalNumberSeparation <= 50) {
+        numberSeparation = 50;
+    } else if (minimalNumberSeparation <= 100) {
+        numberSeparation = 100;
+    } else if (minimalNumberSeparation <= 200) {
+        numberSeparation = 200;
+    } else if (minimalNumberSeparation <= 500) {
+        numberSeparation = 500;
+    } else {
+        numberSeparation = qRound(qreal(numberSeparation / 100.0)) * 100;
     }
-    textLength += 4;  // Add some padding
+    // the chosen separation converted to pixel
+    const int pixelSeparation = qRound(d->viewConverter->documentToViewY(d->unit.fromUserValue(numberSeparation)));
 
-    if (numberStepPixel == 0 || numberStep == 0)
-        return;
-    // Change number step so all digits will fit
-    while(textLength > numberStepPixel) {
-        numberStepPixel += numberStepPixel;
-        numberStep += numberStep;
+    // Draw the marks
+    const int rulerEnd = rulerLengthPixel + d->offset;
+    qreal numberStart = d->unit.toUserValue(d->viewConverter->viewToDocumentY(
+                rectangle.top() - d->offset));
+    // draw the partly hidden mark when the bottom part of the ruler is
+    // invisible
+    int pixelEnd = rectangle.bottom() - d->offset;
+    if (rulerEnd - textLength > rectangle.bottom()) {
+        pixelEnd += textLength;
     }
+    qreal numberEnd = d->unit.toUserValue(d->viewConverter->viewToDocumentX(
+                pixelEnd));
+    if (d->offset < 0) {
+        numberStart -= fmod(numberStart, numberSeparation);
+    }
+    for(qreal n = numberStart; n < numberEnd; n += numberSeparation) {
+        int posOnRuler = qRound(d->viewConverter->documentToViewY(d->unit.fromUserValue(n)));
+        int y = d->offset + posOnRuler;
 
-    // Calc the first number step
-    const int start = d->offset < 0 ? qAbs(d->offset) : 0;
+        // to draw the primary mark
+        QString numberText = QString::number(n);
+        painter.save();
+        painter.translate(rectangle.right() - fullStepMarkerLength, y);
+        painter.drawLine(QPointF(0, 0),
+                QPointF(rectangle.right() - fullStepMarkerLength, 0));
+        painter.setPen(numberPen);
+        painter.rotate(-90);
+        painter.drawText(QPointF(-fontMetrics.width(numberText) / 2.0, 0),
+                numberText);
+        painter.restore();
 
-    // make a little hack so rulers shows correctly inversed number aligned
-    int stepCount = (start / numberStepPixel) + 1;
-    int halfStepCount = (start / qRound(numberStepPixel * 0.5)) + 1;
-    int quarterStepCount = (start / qRound(numberStepPixel * 0.25)) + 1;
+        // start to draw secondary marks following the primary mark
+        painter.setPen(markerPen);
 
-    const QPen numberPen(d->ruler->palette().color(QPalette::Text));
-    const QPen markerPen(d->ruler->palette().color(QPalette::Inactive, QPalette::Text));
-    painter.setPen(markerPen);
+        int yQuarterMark1 = qRound(qreal(y + pixelSeparation / 4.0));
+        if (yQuarterMark1 > rulerEnd)
+            break;
+        painter.drawLine(QPointF(rectangle.right() - 1, yQuarterMark1),
+                QPointF(rectangle.right() - quarterStepMarkerLength, yQuarterMark1));
 
-    if(d->offset > 0)
-        painter.translate(0, d->offset);
+        int yHalfMark = qRound(qreal(y + pixelSeparation / 2.0));
+        if (yHalfMark > rulerEnd)
+            break;
+        painter.drawLine(QPointF(rectangle.right() - 1, yHalfMark),
+                QPointF(rectangle.right() - halfStepMarkerLength, yHalfMark));
 
-    const int len = qRound(rectangle.height()) + start;
-    int nextStep = qRound(d->viewConverter->documentToViewY(
-        d->unit.fromUserValue(numberStep * stepCount)));
-    int nextHalfStep = qRound(d->viewConverter->documentToViewY(d->unit.fromUserValue(
-        numberStep * 0.5 * halfStepCount)));
-    int nextQuarterStep = qRound(d->viewConverter->documentToViewY(d->unit.fromUserValue(
-        numberStep * 0.25 * quarterStepCount)));
-
-    int pos = 0;
-    for(int i = start; i < len; ++i) {
-        pos = i - start;
-
-        if(i == nextStep) {
-            painter.save();
-            painter.translate(rectangle.right()-fullStepMarkerLength, pos);
-            if(pos != 0)
-                painter.drawLine(QPointF(0, 0), QPointF(fullStepMarkerLength-1, 0));
-
-            painter.rotate(-90);
-            int number = qRound(stepCount * numberStep);
-            QString numberText = QString::number(number);
-            painter.setPen(numberPen);
-            painter.drawText(QPointF(-fontMetrics.width(numberText) / 2.0, -measurementTextAboveBelowMargin), numberText);
-            painter.restore();
-
-            ++stepCount;
-            nextStep = qRound(d->viewConverter->documentToViewY(
-                d->unit.fromUserValue(numberStep * stepCount)));
-            ++halfStepCount;
-            nextHalfStep = qRound(d->viewConverter->documentToViewY(d->unit.fromUserValue(
-                numberStep * 0.5 * halfStepCount)));
-            ++quarterStepCount;
-            nextQuarterStep = qRound(d->viewConverter->documentToViewY(d->unit.fromUserValue(
-                numberStep * 0.25 * quarterStepCount)));
-        } else if(i == nextHalfStep) {
-            if(pos != 0)
-                painter.drawLine(QPointF(rectangle.right() - halfStepMarkerLength, pos),
-                                 QPointF(rectangle.right() - 1, pos));
-
-            ++halfStepCount;
-            nextHalfStep = qRound(d->viewConverter->documentToViewY(d->unit.fromUserValue(
-                numberStep * 0.5 * halfStepCount)));
-            ++quarterStepCount;
-            nextQuarterStep = qRound(d->viewConverter->documentToViewY(d->unit.fromUserValue(
-                numberStep * 0.25 * quarterStepCount)));
-        } else if(i == nextQuarterStep) {
-            if(pos != 0)
-                painter.drawLine(QPointF(rectangle.right() - quarterStepMarkerLength, pos),
-                                 QPointF(rectangle.right() - 1, pos));
-
-            ++quarterStepCount;
-            nextQuarterStep = qRound(d->viewConverter->documentToViewY(d->unit.fromUserValue(
-                numberStep * 0.25 * quarterStepCount)));
-        }
+        int yQuarterMark2 = qRound(qreal(y + 3.0 * pixelSeparation / 4.0));
+        if (yQuarterMark2 > rulerEnd)
+            break;
+        painter.drawLine(QPointF(rectangle.right() - 1, yQuarterMark2),
+                QPointF(rectangle.right() - quarterStepMarkerLength, yQuarterMark2));
     }
 
     // Draw the mouse indicator
-    const int mouseCoord = d->mouseCoordinate - start;
+    const int mouseCoord = d->mouseCoordinate + d->offset;
     if (d->selected == KoRulerPrivate::None || d->selected == KoRulerPrivate::HotSpot) {
         const qreal left = rectangle.left() + 1;
         const qreal right = rectangle.right() -1;
-        if (d->selected == KoRulerPrivate::None && d->showMousePosition && mouseCoord > 0 && mouseCoord < rectangle.height() )
+        if (d->selected == KoRulerPrivate::None
+                && d->showMousePosition
+                && d->mouseCoordinate > 0
+                && d->mouseCoordinate < rulerLengthPixel)
             painter.drawLine(QPointF(left, mouseCoord), QPointF(right, mouseCoord));
         foreach (const KoRulerPrivate::HotSpotData & hp, d->hotspots) {
             const qreal y = d->viewConverter->documentToViewY(hp.position) + d->offset;
@@ -611,7 +619,7 @@ void HorizontalDistancesPaintingStrategy::drawDistanceLine(const KoRulerPrivate
 
     painter.save();
     painter.translate(d->offset, d->ruler->height() / 2);
-    painter.setPen(d->ruler->palette().color(QPalette::Text));
+    painter.setPen(QPen(d->ruler->palette().color(QPalette::Text), 0));
     painter.setBrush(d->ruler->palette().color(QPalette::Text));
 
     QLineF line(QPointF(d->viewConverter->documentToViewX(start), 0),
@@ -659,7 +667,7 @@ void HorizontalDistancesPaintingStrategy::drawMeasurements(const KoRulerPrivate
     points << d->effectiveActiveRangeStart();
     points << d->effectiveActiveRangeEnd();
     points << d->rulerLength;
-    qSort(points.begin(), points.end());
+    std::sort(points.begin(), points.end());
     QListIterator<qreal> i(points);
     i.next();
     while (i.hasNext() && i.hasPrevious()) {
@@ -701,7 +709,8 @@ KoRulerPrivate::KoRulerPrivate(KoRuler *parent, const KoViewConverter *vc, Qt::O
     distancesPaintingStrategy((PaintingStrategy*)new HorizontalDistancesPaintingStrategy()),
     paintingStrategy(normalPaintingStrategy),
     ruler(parent),
-    guideCreationStarted(false)
+    guideCreationStarted(false),
+    pixelStep(100.0)
 {
 }
 
@@ -724,7 +733,7 @@ qreal KoRulerPrivate::numberStepForUnit() const
             return 10.0;
         case KoUnit::Point:
         default:
-            return 100.0;
+            return pixelStep;
     }
 }
 
@@ -1042,7 +1051,7 @@ void KoRuler::updateTabs(const QList<KoRuler::Tab> &tabs, qreal tabDistance)
 QList<KoRuler::Tab> KoRuler::tabs() const
 {
     QList<Tab> answer = d->tabs;
-    qSort(answer.begin(), answer.end(), compareTabs);
+    std::sort(answer.begin(), answer.end(), compareTabs);
 
     return answer;
 }
@@ -1360,3 +1369,13 @@ void KoRuler::createGuideToolConnection(KoCanvasBase *canvas)
     connect(this, SIGNAL(guideLineCreated(Qt::Orientation,qreal)),
         tool, SLOT(createGuideLine(Qt::Orientation,qreal)));
 }
+
+void KoRuler::setUnitPixelMultiple2(bool enabled)
+{
+    if (enabled) {
+        d->pixelStep = 64.0;
+    }
+    else {
+        d->pixelStep = 100.0;
+    }
+}
