@@ -57,6 +57,8 @@ void TimelineModelTest::init()
     //m_nodeModel = new KisNodeModel(0);
 
     initBase();
+
+    m_doc->setCurrentImage(m_image);
 }
 
 void TimelineModelTest::cleanup()
@@ -147,6 +149,11 @@ struct TestingInterface : TimelineFramesModel::NodeManipulationInterface
             new KisImageLayerRemoveCommand(m_image, node));
     }
 
+    bool setNodeProperties(KisNodeSP, KisImageSP, KisBaseNode::PropertyList) const override
+    {
+        return false;
+    }
+
 private:
     KisImageSP m_image;
 };
