@@ -109,11 +109,7 @@ double KDcraw::Private::progressValue() const
 
 void KDcraw::Private::fillIndentifyInfo(LibRaw* const raw, DcrawInfoContainer& identify)
 {
-#if QT_VERSION >= 0x050900
     identify.dateTime.setSecsSinceEpoch(raw->imgdata.other.timestamp);
-#else
-    identify.dateTime.setTime_t(raw->imgdata.other.timestamp);
-#endif
     identify.make             = QString(raw->imgdata.idata.make);
     identify.model            = QString(raw->imgdata.idata.model);
     identify.owner            = QString(raw->imgdata.other.artist);
