@@ -19,7 +19,7 @@
 #ifndef KIS_DEFORM_PAINTOP_SETTINGS_H_
 #define KIS_DEFORM_PAINTOP_SETTINGS_H_
 
-
+#include <QScopedPointer>
 #include <brushengine/kis_paintop_settings.h>
 #include <kis_types.h>
 #include <kis_outline_generation_policy.h>
@@ -29,12 +29,20 @@ class KisDeformPaintOpSettings : public KisOutlineGenerationPolicy<KisPaintOpSet
 
 public:
     KisDeformPaintOpSettings();
+    ~KisDeformPaintOpSettings() override;
 
-    QPainterPath brushOutline(const KisPaintInformation &info, OutlineMode mode) const;
+    void setPaintOpSize(qreal value) override;
+    qreal paintOpSize() const override;
 
-    bool paintIncremental();
-    bool isAirbrushing() const;
-    int rate() const;
+    QPainterPath brushOutline(const KisPaintInformation &info, const OutlineMode &mode) override;
 
+    bool paintIncremental() override;
+    bool isAirbrushing() const override;
+
+    QList<KisUniformPaintOpPropertySP> uniformProperties(KisPaintOpSettingsSP settings) override;
+
+private:
+    struct Private;
+    const QScopedPointer<Private> m_d;
 };
 #endif
